#!/bin/sh
#
# Transponder Analyzer for different LNB types
# 
# This script handles different LNB local oscillator frequencies
# and properly calculates tuning frequencies for C-band and Ku-band reception
#

# Set the output directory
OUTPUT_DIR="/tmp"

# Process Options
STOP_ENIGMA2=1  # Default: stop Enigma2
REPORT_FORMAT="both"  # Default: both formats (human readable + JSON)

# Check for no-stop option (for use with properly written frontend management plugins)
if [ "$1" = "--no-stop" ]; then
    STOP_ENIGMA2=0
    shift  # Remove the option from the argument list
fi

# Check for report format options
if [ "$1" = "--json-only" ]; then
    REPORT_FORMAT="json"
    shift  # Remove the option from the argument list
elif [ "$1" = "--text-only" ]; then
    REPORT_FORMAT="text"
    shift  # Remove the option from the argument list
fi

if [ "$1" = "--manual" ]; then
    # Manual entry mode
    echo "==================================================="
    echo "Transponder Analyzer - Manual Entry Mode"
    echo "==================================================="
    
    # Logging for diagnosis
    LOG_FILE="${OUTPUT_DIR}/manual_entry_log.txt"
    echo "Manual mode started at $(date)" > "$LOG_FILE"
    
    # Check if data is piped in or if we need to prompt
    if [ -t 0 ]; then
        # Terminal input - interactive mode
        # Get true frequency
        echo -n "Enter true downlink frequency in MHz (e.g., 4139 for C-band, 11374 for Ku-band): "
        read TRUE_FREQ_MHZ
        
        # Default if empty
        if [ -z "$TRUE_FREQ_MHZ" ]; then
            TRUE_FREQ_MHZ=4139
        fi
        
        # Get polarization
        echo -n "Enter polarization (0=horizontal, 1=vertical): "
        read POLARITY
        
        # Default if empty
        if [ -z "$POLARITY" ]; then
            POLARITY=0
        fi
        
        # Detect band type based on frequency
        if [ "$TRUE_FREQ_MHZ" -lt 5000 ]; then
            # C-band
            echo -n "Enter C-band LNB local oscillator frequency in MHz [5150]: "
            read LO_FREQ_MHZ
            
            # Default if empty
            if [ -z "$LO_FREQ_MHZ" ]; then
                LO_FREQ_MHZ=5150
            fi
        else
            # Ku-band
            echo "Select Ku-band LNB local oscillator frequency:"
            echo "1. 9750 MHz (Universal LNB, low band)"
            echo "2. 10600 MHz (Universal LNB, high band)"
            echo "3. 10750 MHz (DBS LNB)"
            echo -n "Choose LNB type [1]: "
            read LNB_TYPE
            
            # Default if empty
            if [ -z "$LNB_TYPE" ] || [ "$LNB_TYPE" = "1" ]; then
                LO_FREQ_MHZ=9750
            elif [ "$LNB_TYPE" = "2" ]; then
                LO_FREQ_MHZ=10600
            elif [ "$LNB_TYPE" = "3" ]; then
                LO_FREQ_MHZ=10750
            else
                # Default to 9750 for invalid entries
                echo "Invalid selection, using 9750 MHz"
                LO_FREQ_MHZ=9750
            fi
        fi
        
        # Get system
        echo -n "Enter system (0=DVB-S, 1=DVB-S2) [1]: "
        read SYSTEM
        
        # Default if empty
        if [ -z "$SYSTEM" ]; then
            SYSTEM=1
        fi
        
        # Get symbol rate
        echo -n "Enter symbol rate in symbols/sec [27500000]: "
        read SYMBOL_RATE
        
        # Default if empty
        if [ -z "$SYMBOL_RATE" ]; then
            SYMBOL_RATE=27500000
        fi
        
        # Get modulation
        echo -n "Enter modulation (0=QPSK, 1=8PSK) [0]: "
        read MODULATION
        
        # Default if empty
        if [ -z "$MODULATION" ]; then
            MODULATION=0
        fi
        
        # Get FEC
        echo -n "Enter FEC (0=auto, 1=1/2, 2=2/3, 3=3/4, 4=5/6, 5=7/8) [0]: "
        read FEC
        
        # Default if empty
        if [ -z "$FEC" ]; then
            FEC=0
        fi
        
        # Get report format (optional)
        echo -n "Enter report format (both, text, json) [both]: "
        read REPORT_FORMAT_INPUT
        if [ -n "$REPORT_FORMAT_INPUT" ]; then
            REPORT_FORMAT="$REPORT_FORMAT_INPUT"
        fi
    else
        # Non-interactive mode (data piped in) - read from stdin line by line
        echo "Non-interactive mode detected - reading parameters from stdin" >> "$LOG_FILE"
        
        # Read each parameter line by line
        read TRUE_FREQ_MHZ
        read POLARITY
        
        # For LO frequency, check band type
        if [ "$TRUE_FREQ_MHZ" -lt 5000 ]; then
            # C-band - read LO directly
            read LO_FREQ_MHZ
        else
            # Ku-band - read option number and convert
            read LNB_TYPE
            case "$LNB_TYPE" in
                1) LO_FREQ_MHZ=9750 ;;
                2) LO_FREQ_MHZ=10600 ;;
                3) LO_FREQ_MHZ=10750 ;;
                *) LO_FREQ_MHZ=9750 ;;
            esac
        fi
        
        # Read remaining parameters
        read SYSTEM
        read SYMBOL_RATE
        read MODULATION
        read FEC
        # Optional 8th parameter for report format
        read REPORT_FORMAT_INPUT
        if [ -n "$REPORT_FORMAT_INPUT" ]; then
            REPORT_FORMAT="$REPORT_FORMAT_INPUT"
        fi
        
        # Log the parameters
        echo "From stdin - Frequency: $TRUE_FREQ_MHZ" >> "$LOG_FILE"
        echo "From stdin - Polarity: $POLARITY" >> "$LOG_FILE"
        echo "From stdin - LO Frequency: $LO_FREQ_MHZ" >> "$LOG_FILE"
        echo "From stdin - System: $SYSTEM" >> "$LOG_FILE"
        echo "From stdin - Symbol Rate: $SYMBOL_RATE" >> "$LOG_FILE"
        echo "From stdin - Modulation: $MODULATION" >> "$LOG_FILE"
        echo "From stdin - FEC: $FEC" >> "$LOG_FILE"
        
        # Apply default values for any missing parameters
        if [ -z "$TRUE_FREQ_MHZ" ]; then TRUE_FREQ_MHZ=4139; fi
        if [ -z "$POLARITY" ]; then POLARITY=0; fi
        if [ -z "$LO_FREQ_MHZ" ]; then 
            if [ "$TRUE_FREQ_MHZ" -lt 5000 ]; then
                LO_FREQ_MHZ=5150
            else
                LO_FREQ_MHZ=9750
            fi
        fi
        if [ -z "$SYSTEM" ]; then SYSTEM=1; fi
        if [ -z "$SYMBOL_RATE" ]; then SYMBOL_RATE=27500000; fi
        if [ -z "$MODULATION" ]; then MODULATION=0; fi
        if [ -z "$FEC" ]; then FEC=0; fi
    fi
    
    # Log the final values
    echo "Final values used:" >> "$LOG_FILE"
    echo "Frequency: $TRUE_FREQ_MHZ" >> "$LOG_FILE"
    echo "Polarity: $POLARITY" >> "$LOG_FILE"
    echo "LO Frequency: $LO_FREQ_MHZ" >> "$LOG_FILE"
    echo "System: $SYSTEM" >> "$LOG_FILE"
    echo "Symbol Rate: $SYMBOL_RATE" >> "$LOG_FILE"
    echo "Modulation: $MODULATION" >> "$LOG_FILE"
    echo "FEC: $FEC" >> "$LOG_FILE"
else
    # Check arguments for standard mode
    if [ $# -lt 2 ]; then
        echo "Transponder Analyzer"
        echo "Usage: $0 [--no-stop] <true_freq_mhz> <polarity> [lo_freq_mhz]"
        echo "   or: $0 [--no-stop] --manual"
        echo ""
        echo "Options:"
        echo "  --no-stop        Do not stop Enigma2 (for use with frontend-aware plugins)"
        echo ""
        echo "Examples:"
        echo "  $0 4139 0        (Uses default 5150 MHz LO for C-band)"
        echo "  $0 4139 0 5750   (Uses 5750 MHz LO for C-band)"
        echo "  $0 11374 0 9750  (Uses 9750 MHz LO for Ku-band)"
        echo "  $0 12074 1 10600 (Uses 10600 MHz LO for Ku-band)"
        echo "  $0 12450 1 10750 (Uses 10750 MHz LO for Ku-band)"
        echo "  $0 --manual      (Enter all parameters manually)"
        echo "  $0 --no-stop 4139 0  (Don't stop Enigma2 during scan)"
        exit 1
    fi
    
    # Parse command line arguments
    TRUE_FREQ_MHZ=$1
    POLARITY=$2
    
    # Set default LO frequency based on band
    if [ "$TRUE_FREQ_MHZ" -lt 5000 ]; then
        # C-band default
        DEFAULT_LO=5150
    elif [ "$TRUE_FREQ_MHZ" -lt 11700 ]; then
        # Ku-band low
        DEFAULT_LO=9750
    else
        # Ku-band high
        DEFAULT_LO=10600
    fi
    
    LO_FREQ_MHZ=${3:-$DEFAULT_LO}  # Use detected default if not specified
    
    # Set default parameters for pid_extractor_poc
    SYSTEM=1          # Default: DVB-S2
    SYMBOL_RATE=27500000  # Default: 27.5 Msym/s
    MODULATION=0      # Default: QPSK
    FEC=0             # Default: AUTO
fi

# Convert MHz to kHz for the true frequency
TRUE_FREQ_KHZ=$((TRUE_FREQ_MHZ * 1000))
LO_FREQ_KHZ=$((LO_FREQ_MHZ * 1000))

# Calculate tuning frequency based on band
if [ "$TRUE_FREQ_MHZ" -lt 5000 ]; then
    # C-band uses low side LO
    # For C-band: tuning_freq = LO_freq - true_freq
    TUNING_FREQ_KHZ=$((LO_FREQ_KHZ - TRUE_FREQ_KHZ))
    BAND_TYPE="cband"
else
    # Ku-band uses high side LO
    # For Ku-band: tuning_freq = true_freq - LO_freq
    TUNING_FREQ_KHZ=$((TRUE_FREQ_KHZ - LO_FREQ_KHZ))
    BAND_TYPE="kuband"
fi

# Set polarization letter for easier display and compatibility with Python tools
POLARITY_LETTER=$([ "$POLARITY" = "0" ] && echo "H" || echo "V")

# Calculate symbol rate in kSymbols/sec for the Python tools
SYMBOL_RATE_KS=$((SYMBOL_RATE / 1000))

# Generate timestamp for filenames
TIMESTAMP=$(date +"%Y-%m-%d_%H-%M-%S")

# Create output filenames
PARAMS_FILE="${OUTPUT_DIR}/${BAND_TYPE}_params_${TRUE_FREQ_MHZ}MHz_${TIMESTAMP}.txt"
RESULTS_FILE="${OUTPUT_DIR}/${BAND_TYPE}_results_${TRUE_FREQ_MHZ}MHz_${TIMESTAMP}.txt"

# Save parameters to file
{
    echo "==================================================="
    if [ "$BAND_TYPE" = "cband" ]; then
        echo "C-Band Transponder Analyzer Parameters"
    else
        echo "Ku-Band Transponder Analyzer Parameters"
    fi
    echo "==================================================="
    echo "True downlink frequency: $TRUE_FREQ_KHZ kHz ($TRUE_FREQ_MHZ MHz)"
    echo "Polarization: $([ "$POLARITY" = "0" ] && echo "Horizontal" || echo "Vertical")"
    echo "LNB LO frequency: $LO_FREQ_KHZ kHz ($LO_FREQ_MHZ MHz)"
    echo "Calculated tuning frequency: $TUNING_FREQ_KHZ kHz"
    echo "System: $([ "$SYSTEM" = "0" ] && echo "DVB-S" || echo "DVB-S2")"
    echo "Symbol rate: $SYMBOL_RATE symbols/sec"
    echo "Modulation: $([ "$MODULATION" = "0" ] && echo "QPSK" || echo "8PSK")"
    case $FEC in
        0) echo "FEC: AUTO" ;;
        1) echo "FEC: 1/2" ;;
        2) echo "FEC: 2/3" ;;
        3) echo "FEC: 3/4" ;;
        4) echo "FEC: 5/6" ;;
        5) echo "FEC: 7/8" ;;
        6) echo "FEC: 8/9" ;;
        7) echo "FEC: 3/5" ;;
        8) echo "FEC: 4/5" ;;
        9) echo "FEC: 9/10" ;;
        *) echo "FEC: Unknown" ;;
    esac
    echo "Timestamp: $(date)"
    echo "==================================================="
} > "$PARAMS_FILE"

echo "==================================================="
if [ "$BAND_TYPE" = "cband" ]; then
    echo "C-Band Transponder Analyzer"
else
    echo "Ku-Band Transponder Analyzer"
fi
echo "==================================================="
echo "True downlink frequency: $TRUE_FREQ_KHZ kHz ($TRUE_FREQ_MHZ MHz)"
echo "Polarization: $([ "$POLARITY" = "0" ] && echo "Horizontal" || echo "Vertical")"
echo "LNB LO frequency: $LO_FREQ_KHZ kHz ($LO_FREQ_MHZ MHz)"
echo "Calculated tuning frequency: $TUNING_FREQ_KHZ kHz"
echo "System: $([ "$SYSTEM" = "0" ] && echo "DVB-S" || echo "DVB-S2")"
echo "Symbol rate: $SYMBOL_RATE symbols/sec"
echo "Modulation: $([ "$MODULATION" = "0" ] && echo "QPSK" || echo "8PSK")"
case $FEC in
    0) echo "FEC: AUTO" ;;
    1) echo "FEC: 1/2" ;;
    2) echo "FEC: 2/3" ;;
    3) echo "FEC: 3/4" ;;
    4) echo "FEC: 5/6" ;;
    5) echo "FEC: 7/8" ;;
    6) echo "FEC: 8/9" ;;
    7) echo "FEC: 3/5" ;;
    8) echo "FEC: 4/5" ;;
    9) echo "FEC: 9/10" ;;
    *) echo "FEC: Unknown" ;;
esac
echo ""
echo "Parameters saved to: $PARAMS_FILE"
echo ""

# Find pid_extractor_poc - check in standard locations
PID_EXTRACTOR=""
if [ -x "./pid_extractor_poc" ]; then
    PID_EXTRACTOR="./pid_extractor_poc"
elif [ -x "/usr/bin/pid_extractor_poc" ]; then
    PID_EXTRACTOR="/usr/bin/pid_extractor_poc"
elif [ -x "/bin/pid_extractor_poc" ]; then
    PID_EXTRACTOR="/bin/pid_extractor_poc"
elif [ -x "/usr/local/bin/pid_extractor_poc" ]; then
    PID_EXTRACTOR="/usr/local/bin/pid_extractor_poc"
elif command -v pid_extractor_poc >/dev/null 2>&1; then
    PID_EXTRACTOR="pid_extractor_poc"
else
    echo "Error: pid_extractor_poc tool not found in standard locations."
    # For debugging, let's see what's in /usr/bin
    echo "Contents of /usr/bin (grep for pid):"
    ls -la /usr/bin | grep -i pid
    exit 1
fi

# Check for existing frontend information in non-stop mode
ADAPTER=""
FRONTEND=""
if [ $STOP_ENIGMA2 -eq 0 ]; then
    # In non-stop mode, we will check if frontend info was provided
    if [ -f "${OUTPUT_DIR}/active_frontend.txt" ]; then
        echo "Reading frontend information from active_frontend.txt"
        ADAPTER=$(cat "${OUTPUT_DIR}/active_frontend.txt" | grep "adapter=" | cut -d= -f2)
        FRONTEND=$(cat "${OUTPUT_DIR}/active_frontend.txt" | grep "frontend=" | cut -d= -f2)
        echo "Using adapter $ADAPTER, frontend $FRONTEND from configuration"
    fi
fi

# Conditionally stop Enigma2 to free up the frontend
if [ $STOP_ENIGMA2 -eq 1 ]; then
    echo "Stopping Enigma2 service to free frontend..."
    echo "==================================================="
    echo "NOTICE: For security reasons, this script no longer"
    echo "automatically stops Enigma2. If needed, please run:"
    echo "init 4"
    echo "before running this script in terminal mode."
    echo "==================================================="
    # No longer run init 4 automatically
    echo "Automatic init 4 disabled for safety" >> "$LOG_FILE"
    sleep 2
else
    echo "Not stopping Enigma2 (frontend management handled by calling application)..."
    
    # Add a delay to ensure the frontend is released by the calling app
    echo "Waiting for frontend to be released..."
    sleep 3
    
    # Try to force-reset the frontend device
    if [ -n "$ADAPTER" ] && [ -n "$FRONTEND" ]; then
        echo "Attempting to reset frontend device..."
        FRONTEND_DEV="/dev/dvb/adapter${ADAPTER}/frontend${FRONTEND}"
        
        # Try to open and close the device to reset it
        if [ -e "$FRONTEND_DEV" ]; then
            # Close any hanging processes using this device
            fuser -k "$FRONTEND_DEV" 2>/dev/null || true
            echo "Frontend device path: $FRONTEND_DEV"
        fi
    fi
    
    # Additional delay after reset
    sleep 1
fi

# Start redirecting output to results file
{
    echo "==================================================="
    echo "C-Band Transponder Analysis Results"
    echo "==================================================="
    echo "Analysis started at: $(date)"
    echo ""
    
    # Try adapters and frontends
    SUCCESS=0
    
    # If specific adapter and frontend were provided (non-stop mode)
    if [ -n "$ADAPTER" ] && [ -n "$FRONTEND" ]; then
        echo ""
        echo "Using specified adapter $ADAPTER, frontend $FRONTEND..."
        
        # Log command that will be executed
        echo "Running command: $PID_EXTRACTOR $TUNING_FREQ_KHZ $POLARITY $ADAPTER $FRONTEND $SYSTEM $SYMBOL_RATE $MODULATION $FEC"
        
        # Try to force-close the device one more time directly before execution
        if [ -e "/dev/dvb/adapter${ADAPTER}/frontend${FRONTEND}" ]; then
            echo "Final device reset before execution..."
            fuser -k "/dev/dvb/adapter${ADAPTER}/frontend${FRONTEND}" 2>/dev/null || true
            # Extra sleep to let the system fully release the device
            sleep 2
        fi
        
        # Run the command directly (with much simpler execution) - redirect errors to log
        $PID_EXTRACTOR $TUNING_FREQ_KHZ $POLARITY $ADAPTER $FRONTEND $SYSTEM $SYMBOL_RATE $MODULATION $FEC 2>> "$LOG_FILE"
        EXIT_STATUS=$?
        # Check if it was killed by the timeout
        if [ $EXIT_STATUS -eq 137 ]; then
            echo "Command timed out after 30 seconds"
            EXIT_STATUS=124  # Match timeout's exit code
        fi
        
        # We already have EXIT_STATUS from our custom wrapper
        echo "Command exit status: $EXIT_STATUS"
        
        # Check if the command was successful
        if [ $EXIT_STATUS -eq 0 ]; then
            SUCCESS=1
            echo ""
            echo "Success with adapter $ADAPTER, frontend $FRONTEND"
            
            # Check if we have saved signal metrics from the PID Scanner plugin (last 60 seconds)
            METRICS_FILE=$(find "${OUTPUT_DIR}" -name "signal_metrics_*.txt" -mmin -1 | sort -r | head -1)
            if [ -n "$METRICS_FILE" ]; then
                echo ""
                echo "Signal metrics from PID Scanner:"
                SNR=$(grep "SNR=" "$METRICS_FILE" | cut -d= -f2)
                STRENGTH=$(grep "STRENGTH=" "$METRICS_FILE" | cut -d= -f2)
                BER=$(grep "BER=" "$METRICS_FILE" | cut -d= -f2)
                STATUS=$(grep "STATUS=" "$METRICS_FILE" | cut -d= -f2)
                
                echo "  Signal strength: ${STRENGTH}%"
                echo "  SNR: ${SNR} dB"
                echo "  BER: ${BER}"
                echo "  Status: ${STATUS}"
                echo ""
            fi
        elif [ $EXIT_STATUS -eq 124 ]; then
            echo "Command timed out after 30 seconds"
        else
            echo "Command failed with exit code $EXIT_STATUS"
        fi
    else
        # Try different adapters and frontends
        for adapter in 0 1; do
            for frontend in 0 1; do
                echo ""
                echo "Trying adapter $adapter, frontend $frontend..."
                
                # Log command that will be executed
                echo "Running command: $PID_EXTRACTOR $TUNING_FREQ_KHZ $POLARITY $adapter $frontend $SYSTEM $SYMBOL_RATE $MODULATION $FEC"
                
                # Try to force-close the device one more time directly before execution
                if [ -e "/dev/dvb/adapter${adapter}/frontend${frontend}" ]; then
                    echo "Final device reset before execution..."
                    fuser -k "/dev/dvb/adapter${adapter}/frontend${frontend}" 2>/dev/null || true
                    # Extra sleep to let the system fully release the device
                    sleep 2
                fi
                
                # Run the command directly (with much simpler execution) - redirect errors to log
                $PID_EXTRACTOR $TUNING_FREQ_KHZ $POLARITY $adapter $frontend $SYSTEM $SYMBOL_RATE $MODULATION $FEC 2>> "$LOG_FILE"
                EXIT_STATUS=$?
                # Check if it was killed by the timeout
                if [ $EXIT_STATUS -eq 137 ]; then
                    echo "Command timed out after 30 seconds"
                    EXIT_STATUS=124  # Match timeout's exit code
                fi
                
                # We already have EXIT_STATUS from our custom wrapper
                echo "Command exit status: $EXIT_STATUS"
                
                # Check if the command was successful
                if [ $EXIT_STATUS -eq 0 ]; then
                    SUCCESS=1
                    echo ""
                    echo "Success with adapter $adapter, frontend $frontend"
                    
                    # Check if we have saved signal metrics from the PID Scanner plugin (last 60 seconds)
                    METRICS_FILE=$(find "${OUTPUT_DIR}" -name "signal_metrics_*.txt" -mmin -1 | sort -r | head -1)
                    if [ -n "$METRICS_FILE" ]; then
                        echo ""
                        echo "Signal metrics from PID Scanner:"
                        SNR=$(grep "SNR=" "$METRICS_FILE" | cut -d= -f2)
                        STRENGTH=$(grep "STRENGTH=" "$METRICS_FILE" | cut -d= -f2)
                        BER=$(grep "BER=" "$METRICS_FILE" | cut -d= -f2)
                        STATUS=$(grep "STATUS=" "$METRICS_FILE" | cut -d= -f2)
                        
                        echo "  Signal strength: ${STRENGTH}%"
                        echo "  SNR: ${SNR} dB"
                        echo "  BER: ${BER}"
                        echo "  Status: ${STATUS}"
                        echo ""
                    fi
                    
                    break 2
                elif [ $EXIT_STATUS -eq 124 ]; then
                    echo "Command timed out after 30 seconds"
                else
                    echo "Command failed with exit code $EXIT_STATUS"
                fi
            done
        done
    fi
    
    if [ $SUCCESS -eq 0 ]; then
        echo ""
        echo "Analysis failed on all frontends. Try again with different parameters."
    fi
    
    echo ""
    echo "Analysis completed at: $(date)"
    echo "==================================================="
    
    # Add JSON data based on report format
    # Only generate JSON data if in "both" or "json" mode, and never in "text" mode
    if [ "$REPORT_FORMAT" = "both" ] || [ "$REPORT_FORMAT" = "json" ]; then
        if command -v python3 >/dev/null 2>&1; then
            echo "Generating JSON format data..."
            
            # First try the standalone version which is more reliable
            # Check if we have saved signal metrics from the PID Scanner plugin (last 60 seconds)
            METRICS_FILE=$(find "${OUTPUT_DIR}" -name "signal_metrics_*.txt" -mmin -1 | sort -r | head -1)
            SIGNAL_ENV=""
            
            if [ -n "$METRICS_FILE" ]; then
                # Get signal metrics to pass to the Python module
                SNR=$(grep "SNR=" "$METRICS_FILE" | cut -d= -f2)
                STRENGTH=$(grep "STRENGTH=" "$METRICS_FILE" | cut -d= -f2)
                BER=$(grep "BER=" "$METRICS_FILE" | cut -d= -f2)
                STATUS=$(grep "STATUS=" "$METRICS_FILE" | cut -d= -f2)
                
                echo "Using signal metrics from PID Scanner for JSON output"
                SIGNAL_ENV="TSANALYZE_SNR=\"$SNR\" TSANALYZE_STRENGTH=\"$STRENGTH\" TSANALYZE_BER=\"$BER\" TSANALYZE_STATUS=\"$STATUS\""
            fi
            
            if command -v tsanalyze_standalone.py >/dev/null 2>&1; then
                echo "Using standalone tsanalyze..."
                if [ -n "$SIGNAL_ENV" ]; then
                    eval $SIGNAL_ENV python3 $(which tsanalyze_standalone.py) "$TRUE_FREQ_MHZ" "$POLARITY" --json 2>> "$LOG_FILE"
                else
                    python3 $(which tsanalyze_standalone.py) "$TRUE_FREQ_MHZ" "$POLARITY" --json 2>> "$LOG_FILE"
                fi
            elif python3 -c "import importlib.util; print(importlib.util.find_spec('tsanalyze') is not None)" | grep -q "True"; then
                echo "Using module tsanalyze..."
                if [ -n "$SIGNAL_ENV" ]; then
                    eval $SIGNAL_ENV python3 -m tsanalyze "dvb://:${TRUE_FREQ_MHZ}:${POLARITY_LETTER}:${SYMBOL_RATE_KS}" --json 2>> "$LOG_FILE"
                else
                    python3 -m tsanalyze "dvb://:${TRUE_FREQ_MHZ}:${POLARITY_LETTER}:${SYMBOL_RATE_KS}" --json 2>> "$LOG_FILE"
                fi
            else
                echo "Warning: No tsanalyze module found. JSON output will be limited."
                # Generate minimal JSON
                cat << EOF
{
  "transponder": {
    "frequency": ${TRUE_FREQ_MHZ},
    "polarization": "${POLARITY_LETTER}",
    "symbol_rate": ${SYMBOL_RATE},
    "lnb_lo": ${LO_FREQ_MHZ},
    "tuning_frequency": ${TUNING_FREQ_KHZ},
    "system": $([ "$SYSTEM" = "0" ] && echo "\"DVB-S\"" || echo "\"DVB-S2\""),
    "modulation": $([ "$MODULATION" = "0" ] && echo "\"QPSK\"" || echo "\"8PSK\""),
    "timestamp": "$(date +%Y-%m-%d\ %H:%M:%S)",
    "services": []
  }
}
EOF
            fi
        else
            echo "Warning: Python 3 not found. JSON output not available."
            # Generate minimal JSON
            cat << EOF
{
  "transponder": {
    "frequency": ${TRUE_FREQ_MHZ},
    "polarization": "${POLARITY_LETTER}",
    "symbol_rate": ${SYMBOL_RATE},
    "lnb_lo": ${LO_FREQ_MHZ},
    "tuning_frequency": ${TUNING_FREQ_KHZ},
    "system": $([ "$SYSTEM" = "0" ] && echo "\"DVB-S\"" || echo "\"DVB-S2\""),
    "modulation": $([ "$MODULATION" = "0" ] && echo "\"QPSK\"" || echo "\"8PSK\""),
    "timestamp": "$(date +%Y-%m-%d\ %H:%M:%S)",
    "services": []
  }
}
EOF
        fi
    fi
} | tee -a "$RESULTS_FILE" | ( 
    # For JSON-only mode, filter to only show JSON data
    if [ "$REPORT_FORMAT" = "json" ]; then 
        grep -A 9999 "{" 
    # For text-only mode, filter to remove JSON data
    elif [ "$REPORT_FORMAT" = "text" ]; then
        grep -v "^{" | grep -v "    \"" | grep -v "        \"" | grep -v "            \"" | grep -v "                \"" | grep -v "^}" | grep -v "^    }" | grep -v "^        }" | grep -v "^            }"
    # For both formats, show everything
    else
        cat
    fi
)

echo "Results saved to: $RESULTS_FILE"

# If we were in a mode where Enigma2 would have been stopped
if [ $STOP_ENIGMA2 -eq 1 ]; then
    echo ""
    echo "==================================================="
    echo "NOTICE: For security reasons, this script no longer"
    echo "automatically restarts Enigma2. If needed, please run:"
    echo "init 3"
    echo "after running this script in terminal mode."
    echo "==================================================="
    # No longer run init 3 automatically
    echo "Automatic init 3 disabled for safety" >> "$LOG_FILE"
else
    echo ""
    echo "Scan complete - Enigma2 was not stopped"
fi