#!/bin/sh
#
# Current Transponder PID Analyzer
# 
# Extracts PIDs from the currently tuned transponder without stopping Enigma2
# This uses a different approach than other tools by reading directly from the demux

# Get current time for output filenames
TIMESTAMP=$(date +"%Y-%m-%d_%H-%M-%S")

# Output directory
OUTPUT_DIR="/tmp"

# Find the tuner that's currently active
find_active_tuner() {
    # First check all frontends to see which one has a lock
    for adapter in 0 1 2 3; do
        for frontend in 0 1 2 3; do
            if [ -e "/proc/stb/frontend/${adapter}" ]; then
                # Check if this frontend has a lock
                if [ -e "/proc/stb/frontend/${adapter}/lock" ]; then
                    lock_status=$(cat /proc/stb/frontend/${adapter}/lock)
                    if [ "$lock_status" = "1" ]; then
                        echo "Found locked tuner: adapter ${adapter}, frontend ${frontend}"
                        echo "$adapter"
                        return 0
                    fi
                fi
            fi
        done
    done
    
    # If we get here, we didn't find a locked tuner
    echo "No locked tuner found"
    return 1
}

# Extract current transponder info
get_transponder_info() {
    adapter=$1
    
    # Get current frequency
    if [ -e "/proc/stb/frontend/${adapter}/frequency" ]; then
        frequency=$(cat /proc/stb/frontend/${adapter}/frequency)
        echo "Current frequency: $frequency kHz"
    else
        echo "Cannot determine current frequency"
        frequency="unknown"
    fi
    
    # Get current polarization
    if [ -e "/proc/stb/frontend/${adapter}/voltage" ]; then
        voltage=$(cat /proc/stb/frontend/${adapter}/voltage)
        case "$voltage" in
            0) polarity="H"; polarity_val=0 ;;
            1) polarity="V"; polarity_val=1 ;;
            *) polarity="unknown"; polarity_val=0 ;;
        esac
        echo "Current polarization: $polarity"
    else
        echo "Cannot determine current polarization"
        polarity="unknown"
        polarity_val=0
    fi
    
    # Get current symbol rate
    if [ -e "/proc/stb/frontend/${adapter}/symbol_rate" ]; then
        symbol_rate=$(cat /proc/stb/frontend/${adapter}/symbol_rate)
        echo "Current symbol rate: $symbol_rate symbols/sec"
    else
        echo "Cannot determine current symbol rate"
        symbol_rate=27500000
    fi
    
    # Get current delivery system (DVB-S/S2)
    if [ -e "/proc/stb/frontend/${adapter}/delivery_system" ]; then
        delivery_system=$(cat /proc/stb/frontend/${adapter}/delivery_system)
        # Map to DVB-S=0, DVB-S2=1
        case "$delivery_system" in
            0) system=0 ;;  # DVB-S
            9) system=1 ;;  # DVB-S2
            *) system=1 ;;  # Default to DVB-S2
        esac
        echo "Current delivery system: $delivery_system (system=$system)"
    else
        echo "Cannot determine current delivery system"
        system=1  # Default to DVB-S2
    fi
    
    # Try to get modulation and FEC if available
    modulation=0  # Default to AUTO/QPSK
    fec=0         # Default to AUTO
    
    # Output current tuning parameters
    echo "==================================================="
    echo "Current Transponder Parameters:"
    echo "==================================================="
    echo "Frequency: $frequency kHz"
    echo "Polarization: $polarity"
    echo "Symbol Rate: $symbol_rate"
    echo "System: $([ "$system" = "1" ] && echo "DVB-S2" || echo "DVB-S")"
    echo "==================================================="
    
    # Output parameters to a file for future reference
    tp_info_file="${OUTPUT_DIR}/current_tp_info_${TIMESTAMP}.txt"
    {
        echo "Frequency: $frequency kHz"
        echo "Polarization: $polarity"
        echo "Symbol Rate: $symbol_rate"
        echo "System: $([ "$system" = "1" ] && echo "DVB-S2" || echo "DVB-S")"
    } > "$tp_info_file"
    
    # Return values as space-separated string
    echo "$frequency $polarity_val $symbol_rate $system $modulation $fec"
}

# Extract PIDs using pid_extractor_poc
extract_pids() {
    tp_info=$1
    adapter=$2
    
    # Parse transponder info
    freq=$(echo "$tp_info" | cut -d' ' -f1)
    pol=$(echo "$tp_info" | cut -d' ' -f2)
    sr=$(echo "$tp_info" | cut -d' ' -f3)
    sys=$(echo "$tp_info" | cut -d' ' -f4)
    mod=$(echo "$tp_info" | cut -d' ' -f5)
    fec=$(echo "$tp_info" | cut -d' ' -f6)
    
    # Get current time for output files
    output_json="${OUTPUT_DIR}/current_tp_pids_${TIMESTAMP}.json"
    output_txt="${OUTPUT_DIR}/current_tp_pids_${TIMESTAMP}.txt"
    
    echo "Creating temporary filter for PIDs - Enigma2 will still be running!"
    echo "This analyzes the current transponder without retuning."
    
    # Run the PID extractor with special option for current mode
    # We use a special frontend value of 99 to indicate "use currently tuned transponder"
    echo "Running: pid_extractor_poc $freq $pol $adapter 99 $sys $sr $mod $fec"
    pid_extractor_poc "$freq" "$pol" "$adapter" "99" "$sys" "$sr" "$mod" "$fec" > "$output_txt"
    
    # Check if extraction was successful
    if grep -q "JSON Format" "$output_txt"; then
        echo "PID extraction successful!"
        echo "Results saved to:"
        echo "  $output_txt (full text output)"
        echo "  $output_json (JSON data)"
        
        # Extract the JSON part to a separate file
        sed -n '/JSON Format/,/}/p' "$output_txt" | tail -n +2 > "$output_json"
    else
        echo "PID extraction failed."
        echo "Check $output_txt for details."
    fi
}

# Main function
main() {
    echo "Current Transponder PID Analyzer"
    echo "================================="
    
    # Find active tuner
    adapter=$(find_active_tuner)
    if [ $? -ne 0 ]; then
        echo "No active tuner found. Please tune to a channel first."
        exit 1
    fi
    
    # Get transponder info
    tp_info=$(get_transponder_info "$adapter")
    
    # Extract PIDs
    extract_pids "$tp_info" "$adapter"
}

# Run main function
main