#!/usr/bin/env python3
# PID Analyzer - Standalone script to extract real PIDs from scan reports
# This script runs independently from the ServiceScan process
# It stops Enigma2 only once, processes all transponders, then restarts it

import os
import sys
import re
import subprocess
import json
import time
import glob
from datetime import datetime

# Import Enigma2 components for tuning
try:
    from Components.TuneTest import Tuner
    from enigma import eDVBFrontendParametersSatellite, eDVBResourceManager, eTimer
    ENIGMA_IMPORTS_AVAILABLE = True
    print("Enigma2 tuning components imported successfully")
except ImportError:
    ENIGMA_IMPORTS_AVAILABLE = False
    print("WARNING: Enigma2 tuning components not available, falling back to pid_extractor_poc direct method")

# Path where scan reports and blindscan files are stored
SCAN_DIRS = [
    "/media/usb/ServiceScan",
    "/hdd/ServiceScan",
    "/media/FTA/ServiceScan",
    "/tmp",                   # Blindscan results are stored here
    "/usr/share/blindscan"    # Backup location for blindscan files
]

# Default LNB LO frequency for C-band (5750 MHz)
DEFAULT_LO_FREQ = 5750

def get_lnb_lo_frequency(satellite_position, frequency):
    """
    Determine LNB LO frequency based on satellite position and frequency.
    
    Args:
        satellite_position: Orbital position (e.g., "97.0W")
        frequency: Frequency in MHz
    
    Returns:
        LO frequency in MHz
    """
    # Detect band based on frequency
    if frequency and int(frequency) < 4800:
        band_type = 'C'
    elif frequency and 10700 <= int(frequency) <= 12750:
        band_type = 'Ku'
    elif frequency and int(frequency) >= 18000:
        band_type = 'Ka'
    else:
        # Default to C-band for Galaxy 19 (97W) position or if can't determine
        if satellite_position and "97" in satellite_position:
            band_type = 'C'
        else:
            band_type = 'Ku'
    
    # Return default LO frequency for the band
    if band_type == 'C':
        return 5750  # Default to 5750 for C-band
    elif band_type == 'Ku':
        return 9750  # Default to 9750 for Ku-band (low band)
    elif band_type == 'Ka':
        return 21200  # Default for Ka-band
    
    # Unknown band, use safe default
    return DEFAULT_LO_FREQ

def calculate_tuning_frequency(frequency, polarization):
    """
    Calculate the tuning frequency based on band and LNB
    
    Args:
        frequency: Frequency in MHz (can be int or string)
        polarization: 'H' or 'V' or numeric (0=H, 1=V)
    
    Returns:
        Tuple of (tuning_frequency_khz, pol_value)
    """
    # Handle various frequency input formats
    if isinstance(frequency, str) and 'MHz' in frequency:
        frequency = frequency.replace('MHz', '').strip()
    
    # Convert to integer for calculation
    freq_mhz = int(float(frequency))
    
    # Get LNB LO frequency based on frequency band
    lo_freq = get_lnb_lo_frequency(None, freq_mhz)
    
    # Convert frequency to kHz for the calculation
    freq_khz = freq_mhz * 1000
    lo_freq_khz = int(lo_freq) * 1000
    
    # Calculate tuning frequency based on band
    # For C-band (3400-4800 MHz): tuning_freq = LO_freq - true_freq (low side injection)
    # For Ku-band (10700-12750 MHz): tuning_freq = true_freq - LO_freq (high side injection)
    if freq_mhz < 4800:  # C-band
        tuning_freq_khz = lo_freq_khz - freq_khz
        print(f"C-band calculation: {lo_freq_khz} - {freq_khz} = {tuning_freq_khz}")
    else:  # Ku/Ka-band
        tuning_freq_khz = freq_khz - lo_freq_khz
        print(f"Ku/Ka-band calculation: {freq_khz} - {lo_freq_khz} = {tuning_freq_khz}")
        if tuning_freq_khz < 0:
            # This happens if the LO frequency is higher than the signal frequency
            # In this case we need the absolute value as the frontend doesn't use negative frequencies
            tuning_freq_khz = abs(tuning_freq_khz)
            print(f"Negative frequency detected, using absolute value: {tuning_freq_khz}")
    
    # Convert polarization to numeric value (0=H, 1=V)
    if isinstance(polarization, int) or (isinstance(polarization, str) and polarization.isdigit()):
        pol_value = int(polarization)
    else:
        pol_value = 1 if polarization.upper() == 'V' else 0
    
    print(f"Final tuning parameters: frequency={tuning_freq_khz}kHz, polarization={pol_value}")
    return (tuning_freq_khz, pol_value)

def get_known_transponders(satellite_name=None):
    """
    Get a list of known good transponders for the specified satellite
    Based on previous blindscan results
    
    Args:
        satellite_name: Optional name of the satellite to filter by
        
    Returns:
        List of transponder dictionaries with frequency, polarization, symbol_rate, etc.
    """
    # Default strong transponders for Galaxy 19 (97W)
    default_transponders = [
        {
            'frequency': 4079000,  # in kHz
            'polarization': 1,     # 1=vertical, 0=horizontal
            'symbol_rate': 34287000,
            'system': 1,           # 1=DVB-S2, 0=DVB-S
            'modulation': 4,       # 4=QAM_16
            'fec_inner': 2         # FEC values
        },
        {
            'frequency': 4119000,
            'polarization': 1,
            'symbol_rate': 34286000,
            'system': 1,
            'modulation': 4,
            'fec_inner': 0
        },
        {
            'frequency': 4139000,
            'polarization': 0,
            'symbol_rate': 34284000,
            'system': 1,
            'modulation': 4,
            'fec_inner': 4
        }
    ]
    
    # Check for blindscan result files in /tmp and other locations
    blindscan_files = []
    for directory in ['/tmp', '/home/pli/builds/openpli-scarthgap-build-1-30-2025/openpli-oe-core/meta-local/1']:
        if os.path.exists(directory):
            for filename in os.listdir(directory):
                if filename.startswith('blindscan_') and filename.endswith('.xml'):
                    blindscan_files.append(os.path.join(directory, filename))
    
    # If no blindscan files found, return default transponders
    if not blindscan_files:
        print("No blindscan files found, using default transponders")
        return default_transponders
        
    # Get the most recent blindscan file
    latest_file = max(blindscan_files, key=os.path.getmtime)
    print(f"Using transponder data from {latest_file}")
    
    try:
        # Parse the XML file
        import xml.etree.ElementTree as ET
        tree = ET.parse(latest_file)
        root = tree.getroot()
        
        transponders = []
        
        # Extract all transponders from the file
        for sat in root.findall('.//sat'):
            # If satellite name is specified, filter by it
            if satellite_name and satellite_name not in sat.get('name', ''):
                continue
                
            for tp in sat.findall('transponder'):
                transponder = {
                    'frequency': int(tp.get('frequency', 0)),
                    'symbol_rate': int(tp.get('symbol_rate', 0)),
                    'polarization': int(tp.get('polarization', 0)),
                    'system': int(tp.get('system', 1)),
                    'modulation': int(tp.get('modulation', 1)),
                    'fec_inner': int(tp.get('fec_inner', 0))
                }
                transponders.append(transponder)
        
        # If we found transponders, return them
        if transponders:
            # Sort by strongest first (higher symbol rates tend to be stronger)
            transponders.sort(key=lambda x: x['symbol_rate'], reverse=True)
            return transponders
            
    except Exception as e:
        print(f"Error parsing blindscan file: {e}")
    
    # Fallback to default transponders if parsing failed
    return default_transponders

# Global frontend and tuner variables for Enigma2 method
raw_channel = None
frontend = None
tuner = None

def open_frontend(adapter=0):
    """
    Open the frontend using Enigma2 eDVBResourceManager
    
    Args:
        adapter: DVB adapter number
        
    Returns:
        True if successfully opened, False otherwise
    """
    global raw_channel, frontend, tuner
    
    if not ENIGMA_IMPORTS_AVAILABLE:
        print("Not using Enigma2 tuning methods, imports unavailable")
        return False
        
    try:
        # Close any existing frontend
        if frontend is not None or raw_channel is not None:
            print("Closing existing frontend")
            if frontend:
                del frontend
            if raw_channel:
                del raw_channel
            frontend = None
            raw_channel = None
            tuner = None
            
        # Open a new frontend
        res_mgr = eDVBResourceManager.getInstance()
        if res_mgr:
            raw_channel = res_mgr.allocateRawChannel(adapter)
            if raw_channel:
                frontend = raw_channel.getFrontend()
                if frontend:
                    print(f"Successfully opened frontend {adapter}")
                    tuner = Tuner(frontend)
                    return True
                else:
                    print(f"Failed to get frontend for adapter {adapter}")
            else:
                print(f"Failed to allocate raw channel for adapter {adapter}")
        else:
            print("Failed to get eDVBResourceManager instance")
    except Exception as e:
        print(f"Error opening frontend: {str(e)}")
    
    return False

def tune_to_transponder(frequency_khz, polarization, symbol_rate, system=1, modulation=1, fec_inner=0, adapter=0, frontend=0):
    """
    Tune to a real transponder with the specified parameters.
    IMPORTANT: frequency_khz should be the TUNING frequency, not the transponder frequency!
    
    Args:
        frequency_khz: TUNING frequency in kHz (LO-transponder for C-band)
        polarization: 0=horizontal, 1=vertical
        symbol_rate: Symbol rate in symbols/second
        system: 0=DVB-S, 1=DVB-S2
        modulation: Modulation type (1=QPSK, 2=8PSK, etc)
        fec_inner: FEC inner code
        adapter: DVB adapter number
        frontend_num: Frontend number
    
    Returns:
        True if successfully tuned, False otherwise
    """
    # Print detailed transponder information for debugging
    pol_str = "Vertical" if polarization == 1 else "Horizontal"
    sys_str = "DVB-S2" if system == 1 else "DVB-S"
    mod_str = {
        0: "QPSK", 
        1: "QPSK", 
        2: "8PSK", 
        4: "16QAM", 
        5: "32QAM"
    }.get(modulation, f"Unknown({modulation})")
    
    fec_str = {
        0: "AUTO", 
        1: "1/2", 
        2: "2/3", 
        3: "3/4", 
        4: "5/6", 
        5: "7/8", 
        6: "8/9", 
        7: "3/5", 
        8: "4/5", 
        9: "9/10"
    }.get(fec_inner, f"Unknown({fec_inner})")
    
    print("=" * 50)
    print(f"TUNING TO TRANSPONDER WITH THESE PARAMETERS:")
    print(f"Tuning Frequency: {frequency_khz} kHz")
    print(f"Polarization: {pol_str} ({polarization})")
    print(f"Symbol Rate: {symbol_rate} symbols/sec")
    print(f"System: {sys_str} ({system})")
    print(f"Modulation: {mod_str} ({modulation})")
    print(f"FEC: {fec_str} ({fec_inner})")
    print(f"Adapter: {adapter}, Frontend: {frontend}")
    print("=" * 50)
    
    # Try using Enigma2 tuning first if available
    if ENIGMA_IMPORTS_AVAILABLE and tuner is not None:
        try:
            # Create a TuneTest-compatible transponder tuple exactly matching blindscan's approach
            # Format: (0:frequency 1:symbolrate 2:polarisation 3:fec 4:inversion 5:orbpos 6:system 7:modulation 8:rolloff 9:pilot...)
            transponder = [
                frequency_khz // 1000,  # Frequency in MHz for Enigma2
                symbol_rate // 1000,    # Symbol rate in ksym/s for Enigma2
                polarization,           # 0=H, 1=V
                0,                      # FEC_Auto (0)
                0,                      # Inversion_Off (0) - blindscan uses Off instead of Auto!
                192,                    # Orbital position (just a placeholder)
                system,                 # 0=DVB-S, 1=DVB-S2
                0,                      # Modulation_Auto (0)
                0,                      # RollOff_alpha_0_35 (0)
                0,                      # Pilot_Off (0) - blindscan uses Off instead of Auto!
                -1,                     # No_Stream_Id_Filter (-1)
                0,                      # PLS_Gold (0)
                0,                      # PLS_Default_Gold_Code (0)
                -1,                     # No_T2MI_PLP_Id (-1)
                0,                      # T2MI_Default_Pid (0)
                -1,                     # tsid (-1)
                -1                      # onid (-1)
            ]
            
            # Tune using Enigma2's native Tuner class
            print("Tuning with Enigma2 method")
            tuner.tune(transponder)
            
            # Wait for lock
            max_wait = 10  # seconds
            for i in range(max_wait * 2):  # Check every 500ms
                time.sleep(0.5)
                status = {}
                frontend.getFrontendStatus(status)
                if status.get("tuner_state") == "LOCKED":
                    print(f"Successfully tuned to transponder (Enigma2 method)")
                    return True
                elif status.get("tuner_state") == "FAILED":
                    print(f"Frontend reported FAILED state")
                    break
                elif i == (max_wait * 2) - 1:
                    print(f"Timed out waiting for lock")
                    break
            
            print("Enigma2 tuning method failed, falling back to pid_extractor_poc")
        except Exception as e:
            print(f"Error with Enigma2 tuning: {str(e)}")
            print("Falling back to pid_extractor_poc method")
    
    # Fallback to using pid_extractor_poc directly
    try:
        # Build the command to run pid_extractor_poc with the real transponder parameters
        cmd = [
            '/usr/bin/pid_extractor_poc',
            str(frequency_khz),
            str(polarization),
            str(adapter),
            str(frontend),
            str(system),
            str(symbol_rate),
            str(modulation),
            str(fec_inner)
        ]
        
        # Print the exact command we're running for debugging
        print(f"Executing command: {' '.join(cmd)}")
        
        # Call the pid_extractor with real transponder - just to initialize, not to get data
        result = subprocess.call(cmd, stderr=subprocess.STDOUT, timeout=15)
        
        # A return code of 0 means we got a lock and read at least PAT
        if result == 0:
            print(f"Successfully tuned to transponder at {frequency_khz} kHz")
            return True
        else:
            print(f"Failed to tune to transponder at {frequency_khz} kHz")
            return False
            
    except subprocess.TimeoutExpired:
        print(f"Timeout while tuning to transponder at {frequency_khz} kHz")
        return False
    except Exception as e:
        print(f"Error tuning to transponder: {str(e)}")
        return False
        
    return False

def extract_pid_data_for_transponder(frequency, polarization, adapter=0, frontend=0, system=1, 
                                 symbol_rate=27500000, modulation=1, fec_inner=0):
    """
    Run the pid_extractor_poc tool to get accurate PIDs from the live transport stream.
    This function assumes Enigma2 is already stopped.
    
    Args:
        frequency: Frequency in MHz
        polarization: 'H' or 'V'
        adapter: DVB adapter number
        frontend: Frontend number
        system: DVB system (0=DVB-S, 1=DVB-S2)
        symbol_rate: Symbol rate in symbols/second
        modulation: Modulation value (1=QPSK, 2=8PSK, 4=16QAM, etc)
        fec_inner: FEC inner code value
    
    Returns:
        Dictionary with transponder and service information including PIDs
    """
    try:
        # Calculate tuning frequency
        tuning_freq_khz, pol_value = calculate_tuning_frequency(frequency, polarization)
        
        # Print detailed transponder information for PID extraction
        pol_str = "Vertical" if pol_value == 1 else "Horizontal"
        sys_str = "DVB-S2" if system == 1 else "DVB-S"
        mod_str = {
            0: "QPSK", 
            1: "QPSK", 
            2: "8PSK", 
            4: "16QAM", 
            5: "32QAM"
        }.get(modulation, f"Unknown({modulation})")
        
        fec_str = {
            0: "AUTO", 
            1: "1/2", 
            2: "2/3", 
            3: "3/4", 
            4: "5/6", 
            5: "7/8", 
            6: "8/9", 
            7: "3/5", 
            8: "4/5", 
            9: "9/10"
        }.get(fec_inner, f"Unknown({fec_inner})")
        
        print("\n" + "=" * 60)
        print(f"EXTRACTING PIDs FROM TRANSPONDER:")
        print(f"Original Transponder Frequency: {frequency} MHz")
        print(f"Calculated Tuning Frequency: {tuning_freq_khz} kHz")
        print(f"Original Polarization: {polarization}, Tuning Value: {pol_value} ({pol_str})")
        print(f"Symbol Rate: {symbol_rate} symbols/sec")
        print(f"System: {sys_str} ({system})")
        print(f"Modulation: {mod_str} ({modulation})")
        print(f"FEC: {fec_str} ({fec_inner})")
        print(f"Adapter: {adapter}, Frontend: {frontend}")
        print("=" * 60)
        
        # Try multiple retries with adjusted frequencies if needed
        max_retries = 3
        for retry in range(max_retries):
            # Adjust frequency slightly on retries to compensate for tuner drift
            if retry > 0:
                # Adjust tuning frequency by ±1 MHz on subsequent tries
                adjustment = 1000 * retry * (-1 if retry % 2 == 0 else 1)
                adjusted_freq_khz = tuning_freq_khz + adjustment
                print(f"Retry {retry}/{max_retries-1}: Adjusting frequency by {adjustment} kHz to {adjusted_freq_khz} kHz")
            else:
                adjusted_freq_khz = tuning_freq_khz
            
            # Build the command to run pid_extractor_poc with detailed parameters
            cmd = [
                '/usr/bin/pid_extractor_poc',
                str(adjusted_freq_khz),
                str(pol_value),
                str(adapter),
                str(frontend),
                str(system),           # Delivery system flag (0=DVB-S, 1=DVB-S2)
                str(symbol_rate),      # Symbol rate
                str(modulation),       # Modulation (important for DVB-S2)
                str(fec_inner)         # FEC inner code (also important)
            ]
            
            print(f"Executing command: {' '.join(cmd)}")
            
            try:
                # Set timeout to avoid hanging
                result = subprocess.check_output(cmd, stderr=subprocess.STDOUT, universal_newlines=True, timeout=30)
                
                # Extract the JSON data from the output
                json_start = result.find('{')
                json_end = result.rfind('}') + 1
                
                if json_start >= 0 and json_end > json_start:
                    json_data = json.loads(result[json_start:json_end])
                    print(f"Successfully extracted PID data from transponder")
                    return json_data
                else:
                    print(f"No valid JSON data found in PID extractor output, retrying...")
            except subprocess.TimeoutExpired:
                print(f"PID extraction timed out, retrying with different parameters...")
            except subprocess.CalledProcessError as e:
                print(f"PID extraction failed: {e.output}")
                print(f"Retrying with different parameters...")
            except Exception as e:
                print(f"Error: {str(e)}")
                print(f"Retrying with different parameters...")
                
        # If we reach here, all retries failed
        print(f"All extraction attempts failed for {frequency} MHz {polarization}")
        return None
            
    except Exception as e:
        print(f"Error in extract_pid_data_for_transponder: {str(e)}")
        return None

def parse_scan_report(filepath):
    """Extract transponder information from ServiceScan XML report or blindscan XML file"""
    transponders = []
    satellite = ""
    
    # Read the file and parse as needed
    with open(filepath, 'r') as f:
        content = f.read()
        
        # Check if this is a blindscan file (has different XML format)
        if "blindscan" in filepath.lower() or "<satellites>" in content:
            # Parse blindscan XML format
            try:
                # Try to use minidom to parse the XML
                import xml.dom.minidom as minidom
                dom = minidom.parseString(content)
                
                # Get satellite name
                sat_elements = dom.getElementsByTagName("sat")
                if sat_elements and sat_elements[0].hasAttribute("name"):
                    satellite = sat_elements[0].getAttribute("name").strip()
                
                # Extract transponder info
                tp_elements = dom.getElementsByTagName("transponder")
                tp_num = 0
                
                for tp in tp_elements:
                    tp_num += 1
                    if tp.hasAttribute("frequency") and tp.hasAttribute("polarization"):
                        # Get required attributes
                        freq = int(tp.getAttribute("frequency")) // 1000  # Convert kHz to MHz
                        pol_val = int(tp.getAttribute("polarization"))
                        pol = "V" if pol_val == 1 else "H"
                        
                        # Get optional attributes with defaults
                        symbol_rate = int(tp.getAttribute("symbol_rate")) if tp.hasAttribute("symbol_rate") else 27500000
                        system = int(tp.getAttribute("system")) if tp.hasAttribute("system") else 1  # Default DVB-S2
                        modulation = int(tp.getAttribute("modulation")) if tp.hasAttribute("modulation") else 1  # Default QPSK
                        fec_inner = int(tp.getAttribute("fec_inner")) if tp.hasAttribute("fec_inner") else 0  # Default Auto
                        
                        # Create transponder object
                        tp_obj = {
                            'number': tp_num,
                            'frequency': freq,
                            'polarization': pol,
                            'symbol_rate': symbol_rate,
                            'system': system,
                            'modulation': modulation,
                            'fec_inner': fec_inner,
                            'full_info': f"Blindscan {freq}{pol}/{symbol_rate//1000}"
                        }
                        
                        transponders.append(tp_obj)
            except Exception as e:
                print(f"Error parsing blindscan XML: {e}")
                
                # Fallback to regex if XML parsing fails
                # Extract transponder info using regex
                tp_matches = re.findall(r'<transponder\s+frequency="(\d+)"\s+symbol_rate="(\d+)"\s+polarization="(\d+)"[^>]*\s+system="(\d+)"[^>]*\s+modulation="(\d+)"[^>]*\s+fec_inner="(\d+)"', content)
                
                tp_num = 0
                for freq, sr, pol_val, system, modulation, fec in tp_matches:
                    tp_num += 1
                    freq = int(freq) // 1000  # Convert kHz to MHz
                    pol = "V" if int(pol_val) == 1 else "H"
                    
                    tp_obj = {
                        'number': tp_num,
                        'frequency': freq,
                        'polarization': pol,
                        'symbol_rate': int(sr),
                        'system': int(system),
                        'modulation': int(modulation),
                        'fec_inner': int(fec),
                        'full_info': f"Blindscan {freq}{pol}/{int(sr)//1000}"
                    }
                    
                    transponders.append(tp_obj)
                
                # Try to extract satellite name with regex
                sat_match = re.search(r'<sat\s+name="([^"]+)"', content)
                if sat_match:
                    satellite = sat_match.group(1).strip()
        else:
            # Handle regular ServiceScan XML format using regex
            # Get satellite name
            sat_match = re.search(r'<\s*Satellite\s*=\s*([^>]+)\s*>', content)
            if sat_match:
                satellite = sat_match.group(1).strip()
            
            # Extract transponder information - handle both formats
            tp_matches = re.findall(r"<\s*'(DVB-S2?[^']+)'\s*>\s*Tp#\s*(\d+)", content)
            
            # If no matches found, try alternative pattern seen in real reports
            if not tp_matches:
                tp_matches = re.findall(r"<\s*'(DVB-S2\s+\w+\s+\d+[HV]\s*/\s*\d+\s*/\s*[\d/]+)'\s*>\s*Tp#\s*(\d+)", content)
            
            # Process each transponder
            for tp_info, tp_num in tp_matches:
                # Extract frequency and polarization
                freq_match = re.search(r'(\d+)([HV])', tp_info)
                if freq_match:
                    freq = freq_match.group(1)
                    pol = freq_match.group(2)
                    
                    # Look for symbol rate
                    sr_match = re.search(r'/\s*(\d+)', tp_info)
                    symbol_rate = int(sr_match.group(1)) * 1000 if sr_match else 27500000
                    
                    # Detect system type
                    system = 1 if 'DVB-S2' in tp_info else 0
                    
                    # Create transponder object
                    tp_obj = {
                        'number': tp_num,
                        'frequency': freq,
                        'polarization': pol,
                        'symbol_rate': symbol_rate,
                        'system': system,
                        'full_info': tp_info
                    }
                    
                    transponders.append(tp_obj)
    
    return {
        'satellite': satellite,
        'transponders': transponders,
        'filepath': filepath
    }

def create_pid_data_file(scan_data, pid_data):
    """Create a data file with the real PID information"""
    base_path = os.path.dirname(scan_data['filepath'])
    base_name = os.path.basename(scan_data['filepath'])
    
    if base_name.endswith('.xml'):
        pid_file = os.path.join(base_path, base_name.replace('.xml', '.pids.json'))
    else:
        pid_file = os.path.join(base_path, base_name + '.pids.json')
    
    with open(pid_file, 'w') as f:
        json.dump(pid_data, f, indent=2)
    
    print(f"Created PID data file: {pid_file}")
    return pid_file

def analyze_scan_file(scan_file, output_dir=None):
    """Process a scan file and extract PIDs for all transponders"""
    print(f"Processing file: {scan_file}")
    
    # Determine file type (scan report or blindscan)
    file_type = "blindscan" if "blindscan" in scan_file.lower() else "scan report"
    print(f"Detected file type: {file_type.upper()}")
    
    # Parse the scan report or blindscan file
    scan_data = parse_scan_report(scan_file)
    print(f"Found satellite: {scan_data['satellite']}")
    print(f"Found {len(scan_data['transponders'])} transponders")
    
    # Initialize PID data storage
    pid_data = {
        'satellite': scan_data['satellite'],
        'scan_file': os.path.basename(scan_file),
        'timestamp': datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        'transponders': {}
    }
    
    # Verify dependencies
    if not os.path.exists('/usr/bin/pid_extractor_poc') or not os.access('/usr/bin/pid_extractor_poc', os.X_OK):
        raise Exception("pid_extractor_poc not found or not executable")
    
    if not os.path.exists('/usr/bin/enigma2_control.sh') or not os.access('/usr/bin/enigma2_control.sh', os.X_OK):
        raise Exception("enigma2_control.sh not found or not executable")
    
    # Stop Enigma2 once before processing all transponders
    print("Stopping Enigma2 to free up the tuner...")
    os.system('/usr/bin/enigma2_control.sh stop')
    time.sleep(3)  # Give it time to fully stop
    
    try:
        # Initialize the frontend using Enigma2 if available
        if ENIGMA_IMPORTS_AVAILABLE:
            print("Initializing frontend using Enigma2 tuning system")
            if open_frontend(adapter=0):
                print("Successfully opened frontend using Enigma2 method")
            else:
                print("Failed to open frontend using Enigma2 method, will fall back to pid_extractor_poc")
        
        # Get a list of known good transponders from blindscan results
        # We'll use one of these to initialize the tuner before analyzing individual transponders
        known_transponders = get_known_transponders(satellite_name=scan_data['satellite'])
        
        # Find a good initialization transponder with strong signal
        init_success = False
        print("Initializing tuner with a known good transponder...")
        
        # Try the strongest transponders first (they're sorted by symbol rate)
        for tp in known_transponders[:3]:  # Try the top 3 strongest
            # Convert transponder frequency (kHz) to MHz for calculation
            tp_freq_mhz = tp['frequency'] / 1000
            pol_str = "V" if tp['polarization'] == 1 else "H"
            
            print(f"Trying initialization with transponder: {tp_freq_mhz} MHz {pol_str}")
            
            # Calculate tuning frequency - this is what the hardware needs
            tuning_freq_khz, pol_value = calculate_tuning_frequency(tp_freq_mhz, tp['polarization'])
            
            print(f"Original frequency: {tp_freq_mhz} MHz, Calculated tuning frequency: {tuning_freq_khz} kHz")
            
            if tune_to_transponder(
                tuning_freq_khz,  # Pass the actual tuning frequency to the tuner, not the transponder freq 
                tp['polarization'], 
                tp['symbol_rate'],
                tp['system'],
                tp['modulation'],
                tp['fec_inner'],
                adapter=0,
                frontend=0
            ):
                init_success = True
                print(f"Successfully initialized tuner with transponder at {tp_freq_mhz} MHz (tuned to {tuning_freq_khz} kHz)")
                break
                
        if not init_success:
            print("WARNING: Failed to initialize tuner with any known transponders")
            print("Continuing anyway, but PID extraction may fail")
        
        # Process all transponders in a single Enigma2-stopped session
        for tp in scan_data['transponders']:
            print(f"Analyzing transponder {tp['number']}: {tp['frequency']}{tp['polarization']}")
            
            # Find closest matching transponder from our known good list
            # This helps us get the right modulation, fec values, etc.
            matching_tp = None
            min_freq_diff = float('inf')
            
            # Convert scan frequency and polarization for comparison
            try:
                scan_freq = int(tp['frequency'])  # MHz from ServiceScan
                scan_pol = 1 if tp['polarization'].upper() == 'V' else 0
                
                # Try to find a matching transponder from the known good list
                for known_tp in known_transponders:
                    # known_tp freq is in kHz from blindscan, convert to MHz for comparison
                    known_freq_mhz = known_tp['frequency'] / 1000
                    
                    # Calculate frequency difference
                    freq_diff = abs(scan_freq - known_freq_mhz)
                    
                    # Get polarization string for display
                    pol_str = "Vertical" if known_tp['polarization'] == 1 else "Horizontal"
                    
                    # If very close in frequency and same polarization, consider it a match
                    if freq_diff < 1.0 and known_tp['polarization'] == scan_pol:
                        matching_tp = known_tp
                        print(f"Found exact matching transponder: {known_freq_mhz} MHz {pol_str}")
                        break
                    
                    # Keep track of closest match as fallback
                    if freq_diff < min_freq_diff:
                        min_freq_diff = freq_diff
                        matching_tp = known_tp
                
                if matching_tp:
                    print(f"Using transponder parameters: {matching_tp['frequency']/1000} MHz, "
                          f"pol={'V' if matching_tp['polarization']==1 else 'H'}, "
                          f"SR={matching_tp['symbol_rate']}, "
                          f"mod={matching_tp['modulation']}, "
                          f"fec={matching_tp['fec_inner']}")
                else:
                    print("No matching transponder found, using defaults")
            except Exception as e:
                print(f"Error matching transponder: {e}")
                print("Using default parameters")
            
            # Use matching transponder parameters if found, otherwise defaults
            modulation = matching_tp['modulation'] if matching_tp else 1
            fec_inner = matching_tp['fec_inner'] if matching_tp else 0
            
            # Extract PIDs for this transponder with detailed parameters
            try:
                result = extract_pid_data_for_transponder(
                    tp['frequency'], 
                    tp['polarization'],
                    adapter=0,
                    frontend=0,
                    system=tp['system'],
                    symbol_rate=tp['symbol_rate'],
                    modulation=modulation,
                    fec_inner=fec_inner
                )
                
                if result and 'transponder' in result:
                    pid_data['transponders'][f"{tp['frequency']}-{tp['polarization']}"] = result['transponder']
                    print(f"Successfully extracted PIDs for transponder {tp['number']}")
                else:
                    print(f"Failed to extract PIDs for transponder {tp['number']}")
                    
                    # If we failed, try again after reinitializing with a known good transponder
                    print("Retrying after reinitializing tuner with a known good transponder...")
                    
                    # Try to find a strong transponder from our list of known good ones
                    for init_tp in known_transponders[:3]:  # Try top 3 strongest
                        # Convert transponder frequency (kHz) to MHz for calculation
                        init_tp_freq_mhz = init_tp['frequency'] / 1000
                        init_pol_str = "V" if init_tp['polarization'] == 1 else "H"
                        
                        # Calculate tuning frequency for initialization
                        init_tuning_freq_khz, init_pol_value = calculate_tuning_frequency(init_tp_freq_mhz, init_tp['polarization'])
                        
                        print(f"Trying to reinitialize with: {init_tp_freq_mhz} MHz {init_pol_str}")
                        print(f"Calculated tuning frequency: {init_tuning_freq_khz} kHz")
                        
                        if tune_to_transponder(
                            init_tuning_freq_khz,  # Pass the calculated tuning frequency, not the transponder freq
                            init_tp['polarization'], 
                            init_tp['symbol_rate'],
                            init_tp['system'],
                            init_tp['modulation'],
                            init_tp['fec_inner']
                        ):
                            print(f"Reinitialized tuner with transponder at {init_tp_freq_mhz} MHz (tuned to {init_tuning_freq_khz} kHz)")
                            time.sleep(1)  # Let tuner settle
                            break
                    
                    # Try extraction again with the same parameters as before
                    result = extract_pid_data_for_transponder(
                        tp['frequency'], 
                        tp['polarization'],
                        adapter=0,
                        frontend=0,
                        system=tp['system'],
                        symbol_rate=tp['symbol_rate'],
                        modulation=modulation,
                        fec_inner=fec_inner
                    )
                    
                    if result and 'transponder' in result:
                        pid_data['transponders'][f"{tp['frequency']}-{tp['polarization']}"] = result['transponder']
                        print(f"Successfully extracted PIDs for transponder {tp['number']} on second attempt")
                    else:
                        print(f"Failed to extract PIDs for transponder {tp['number']} even after retry")
                
            except Exception as e:
                print(f"Error analyzing transponder {tp['number']}: {e}")
                continue
    finally:
        # Close the frontend if it was opened with Enigma2
        if ENIGMA_IMPORTS_AVAILABLE and (frontend is not None or raw_channel is not None):
            print("Closing Enigma2 frontend")
            if frontend:
                del frontend
            if raw_channel:
                del raw_channel
            if tuner:
                del tuner
            frontend = None
            raw_channel = None
            tuner = None
        
        # Always restart Enigma2 when done, even if an error occurred
        print("Restarting Enigma2...")
        os.system('/usr/bin/enigma2_control.sh start')
    
    # Create PID data file
    pid_file = create_pid_data_file(scan_data, pid_data)
    
    # Create a marker file to indicate successful processing
    with open(f"{scan_file}.pid_analyzed", "w") as f:
        f.write(f"Processed at {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}\n")
    
    # Give proper permissions
    os.chmod(f"{scan_file}.pid_analyzed", 0o644)
    
    print(f"PID analysis completed: {pid_file}")
    return pid_file

def find_latest_scan_file():
    """Find the latest scan file in any of the standard directories"""
    latest_file = None
    latest_time = 0
    
    for dir_path in SCAN_DIRS:
        if not os.path.exists(dir_path):
            continue
            
        # Look for XML scan files
        for file_path in glob.glob(os.path.join(dir_path, "*.xml")):
            # Skip files that have already been processed
            if os.path.exists(f"{file_path}.pid_analyzed"):
                continue
                
            # Check if this is a scan report file or a blindscan file
            is_valid_file = False
            try:
                with open(file_path, 'r') as f:
                    content = f.read(1000)  # Read the first 1000 characters
                    
                    # Check if this is a ServiceScan report
                    if "Scan Report" in content:
                        is_valid_file = True
                    
                    # Check if this is a blindscan file
                    elif "blindscan" in file_path.lower() or "<satellites>" in content:
                        is_valid_file = True
            except Exception as e:
                print(f"Error reading file {file_path}: {e}")
                continue
                
            if not is_valid_file:
                continue
            
            # Check if this is newer than our current latest
            file_time = os.path.getmtime(file_path)
            if file_time > latest_time:
                latest_time = file_time
                latest_file = file_path
    
    return latest_file

def main():
    """Main function to find and process the latest scan file"""
    # Set up logging to a file
    import logging
    log_file = "/tmp/pid_analyzer.log"
    logging.basicConfig(
        filename=log_file,
        level=logging.INFO,
        format='%(asctime)s - %(levelname)s - %(message)s'
    )
    
    # Also log to console
    console = logging.StreamHandler()
    console.setLevel(logging.INFO)
    logging.getLogger('').addHandler(console)
    
    logging.info("Starting pid_analyzer.py")
    
    # Check for command-line argument (specific scan file)
    if len(sys.argv) > 1:
        scan_file = sys.argv[1]
        if not os.path.exists(scan_file):
            logging.error(f"Scan file does not exist: {scan_file}")
            print(f"Error: Scan file does not exist: {scan_file}")
            sys.exit(1)
    else:
        # Find the latest scan file
        scan_file = find_latest_scan_file()
        if not scan_file:
            logging.info("No new scan or blindscan files found for processing")
            print("No new scan or blindscan files found for processing")
            # List all directories that were searched
            print("\nSearched directories:")
            for directory in SCAN_DIRS:
                exists = "exists" if os.path.exists(directory) else "does not exist"
                print(f"  {directory} ({exists})")
            # List all XML files found in /tmp for debugging
            if os.path.exists("/tmp"):
                print("\nXML files in /tmp:")
                for xml_file in glob.glob("/tmp/*.xml"):
                    print(f"  {xml_file} (modified: {datetime.fromtimestamp(os.path.getmtime(xml_file)).strftime('%Y-%m-%d %H:%M:%S')})")
            sys.exit(0)
    
    # Optional output directory parameter
    output_dir = None
    if len(sys.argv) > 2:
        output_dir = sys.argv[2]
        logging.info(f"Using custom output directory: {output_dir}")
    
    try:
        logging.info(f"Processing scan file: {scan_file}")
        analyze_scan_file(scan_file, output_dir)
        logging.info("PID analysis completed successfully")
        
    except Exception as e:
        logging.exception(f"Error processing scan file: {e}")
        print(f"Error processing scan file: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()