from time import strftime, time
import os
import json
import subprocess
import re

from Components.ActionMap import ActionMap
from Components.config import ConfigSelection, ConfigSubsection, ConfigYesNo, ConfigSlider, ConfigInteger, ConfigText, config, getConfigListEntry, NoSave, ConfigNumber, ConfigSet
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.NimManager import nimmanager
from Components.TuneTest import Tuner
from Components.Sources.StaticText import StaticText
from enigma import eDVBResourceManager, eTimer, eDVBFrontendParametersSatellite, gFont
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Tools.Directories import fileExists
from Plugins.Plugin import PluginDescriptor

# Output directory - will be dynamically set based on available directories
PID_DATA_DIR = "/tmp"

# Function to get or create the best output directory
def get_pid_scan_dir(create=False):
    """
    Find the best directory to store scan results in, prioritizing removable media
    
    Args:
        create: If True, attempt to create the PidScan directory if it doesn't exist
               Only creates in locations that exist and are writable
    
    Returns:
        Path to the best available PidScan directory
    """
    default_dir = "/tmp"
    
    # Check locations in order of preference
    locations = [
        "/media/usb",
        "/media/FTA",
        "/hdd"
    ]
    
    try:
        for base_dir in locations:
            scan_dir = os.path.join(base_dir, "PidScan")
            
            # Check if PidScan directory already exists
            if os.path.exists(scan_dir):
                return scan_dir
                
            # If not found and we're asked to create it
            if create and os.path.exists(base_dir) and os.access(base_dir, os.W_OK):
                try:
                    os.mkdir(scan_dir)
                    print(f"Created PidScan directory at {scan_dir}")
                    return scan_dir
                except Exception as e:
                    print(f"Error creating directory {scan_dir}: {e}")
    except Exception as e:
        print(f"Error checking for PidScan directories: {e}")
        
    return default_dir

# Create plugin-wide configuration
config.plugins = ConfigSubsection() if not hasattr(config, "plugins") else config.plugins
config.plugins.pidscanner = ConfigSubsection()
config.plugins.pidscanner.report_format = ConfigSelection(default="human_readable", choices=[
    ("human_readable", _("Human Readable")),
    ("json", _("JSON Only")),
    ("both", _("Both Formats"))
])

# Add option to create PidScan directory automatically
config.plugins.pidscanner.create_dir = ConfigYesNo(default=True)
config.plugins.pidscanner.last_frequency_band = ConfigSelection(default="cband", choices=[
    ("cband", _("C-band")),
    ("kuband", _("Ku-band"))
])
config.plugins.pidscanner.last_cband_lo = ConfigSelection(default="5150", choices=[
    ("5150", _("5150 MHz")),
    ("5750", _("5750 MHz"))
])
config.plugins.pidscanner.last_kuband_lo = ConfigSelection(default="9750", choices=[
    ("9750", _("9750 MHz")),
    ("10600", _("10600 MHz")),
    ("10750", _("10750 MHz"))
])

# Method to call the cband_analyzer.sh script 
def run_transponder_analyzer(freq_mhz, polarity, lo_freq_mhz=None):
    """
    Run the cband_analyzer.sh script (now supports both C-band and Ku-band)
    
    Args:
        freq_mhz: True downlink frequency in MHz
        polarity: 0=horizontal, 1=vertical
        lo_freq_mhz: LNB LO frequency in MHz (optional)
    
    Returns:
        Tuple of (success, output_message)
    """
    try:
        # Build command to run the transponder analyzer
        cmd = ['/usr/bin/cband_analyzer.sh', str(freq_mhz), str(polarity)]
        
        # Add LO frequency if provided
        if lo_freq_mhz is not None:
            cmd.append(str(lo_freq_mhz))
            
        print(f"Running: {' '.join(cmd)}")
        
        # Run the script and capture output
        result = subprocess.check_output(cmd, stderr=subprocess.STDOUT, universal_newlines=True)
        
        # Return success and the output
        return (True, result)
        
    except subprocess.CalledProcessError as e:
        return (False, f"Error running transponder analyzer: {e.output}")
    except Exception as e:
        return (False, f"Error: {str(e)}")

class PIDScanner(Screen, ConfigListScreen):
    # Define a custom skin with LNB info display (1.5x larger)
    skin = """
    <screen name="PIDScanner" position="center,center" size="1350,1020" title="PID Scanner">
        <widget name="config" position="45,75" size="1260,675" scrollbarMode="showOnDemand" font="Regular;30" itemHeight="45" />
        <widget name="lnb_info" position="45,765" size="1260,45" font="Regular;33" foregroundColor="#00ffff00" />
        <widget name="description" position="45,820" size="1260,120" font="Regular;33" valign="center" />
        <ePixmap pixmap="buttons/red.png" position="60,925" size="210,60" alphatest="blend" />
        <ePixmap pixmap="buttons/green.png" position="285,925" size="210,60" alphatest="blend" />
        <widget source="key_red" render="Label" position="60,925" zPosition="1" size="210,60" font="Regular;30" halign="center" valign="center" backgroundColor="#9f1313" foregroundColor="white" transparent="1" />
        <widget source="key_green" render="Label" position="285,925" zPosition="1" size="210,60" font="Regular;30" halign="center" valign="center" backgroundColor="#1f771f" foregroundColor="white" transparent="1" />
        <!-- Required for virtual keyboard and input helpers -->
        <widget name="HelpWindow" position="976,400" zPosition="1" size="1,1" />
        <widget name="VirtualKeyBoard" position="780,300" size="500,400" zPosition="2" backgroundColor="#8f8f8f" transparent="1" alphatest="blend" />
    </screen>
    """
    
    def __init__(self, session):
        Screen.__init__(self, session)
        Screen.setTitle(self, _("PID Scanner"))
        
        # Initialize frontend resources
        self.raw_channel = None
        self.frontend = None
        self.tuner = None
        self.feid = -1
        
        # Create configuration for tuner selection
        nim_list = []
        for n in nimmanager.nim_slots:
            if not n.isCompatible("DVB-S"):
                continue
            if hasattr(n, 'isFBCLink') and n.isFBCLink():
                continue
            if n.config_mode == "nothing":
                continue
            if len(nimmanager.getSatListForNim(n.slot)) < 1:
                continue
            nim_list.append((str(n.slot), n.friendly_full_description))
            
        self.scan_nims = ConfigSelection(choices=nim_list)
        
        # Frequency type selection (C-band or Ku-band) - use last saved value
        self.frequency_band = ConfigSelection(default=config.plugins.pidscanner.last_frequency_band.value, choices=[
            ("cband", _("C-band (3400-4200 MHz)")),
            ("kuband", _("Ku-band (10700-12750 MHz)"))
        ])
        
        # Frequency input - using NoSave to ensure it's properly editable
        # Set default based on band
        default_freq = 4000 if config.plugins.pidscanner.last_frequency_band.value == "cband" else 11300
        self.frequency = NoSave(ConfigNumber(default=default_freq))
        
        # Polarization
        self.polarization = ConfigSelection(default="0", choices=[
            ("0", _("Horizontal")),
            ("1", _("Vertical"))
        ])
        
        # Symbol rate input (in kSymbols/sec)
        self.symbolrate = NoSave(ConfigNumber(default=27500))
        
        # System type
        self.system = ConfigSelection(default="1", choices=[
            ("0", _("DVB-S")),
            ("1", _("DVB-S2"))
        ])
        
        # Modulation
        self.modulation = ConfigSelection(default="0", choices=[
            ("0", _("QPSK")),
            ("1", _("8PSK"))
        ])
        
        # FEC
        self.fec = ConfigSelection(default="0", choices=[
            ("0", _("Auto")),
            ("1", _("1/2")),
            ("2", _("2/3")),
            ("3", _("3/4")),
            ("4", _("5/6")),
            ("5", _("7/8")),
            ("6", _("8/9")),
            ("7", _("3/5")),
            ("8", _("4/5")),
            ("9", _("9/10"))
        ])
        
        # LNB selection
        self.lnb_type = ConfigSelection(default="default", choices=[
            ("default", _("Default LNB")),
            ("manual", _("Manual LO entry"))
        ])
        
        # C-band LNB LO frequencies - use last saved value
        self.cband_lo_freq = ConfigSelection(default=config.plugins.pidscanner.last_cband_lo.value, choices=[
            ("5150", _("5150 MHz")),
            ("5750", _("5750 MHz"))
        ])
        
        # Ku-band LNB LO frequencies - use last saved value
        self.kuband_lo_freq = ConfigSelection(default=config.plugins.pidscanner.last_kuband_lo.value, choices=[
            ("9750", _("9750 MHz (Universal LNB Low)")),
            ("10600", _("10600 MHz (Universal LNB High)")),
            ("10750", _("10750 MHz (DBS LNB)"))
        ])
        
        # Report format selection - use global setting
        self.report_format = ConfigSelection(default=config.plugins.pidscanner.report_format.value, choices=[
            ("human_readable", _("Human Readable Format")),
            ("json", _("JSON Format Only")),
            ("both", _("Both Formats"))
        ])
        
        # Manual LO entry
        self.manual_lo_freq = ConfigInteger(default=5150, limits=(3000, 11000))
        
        # We no longer need to explicitly stop Enigma2 with proper frontend management
        # This is kept for internal use but hidden from the UI
        self.stop_enigma2 = ConfigYesNo(default=False)
        
        self["actions"] = ActionMap(["SetupActions", "ColorActions", "DirectionActions"],
        {
            "cancel": self.keyCancel,
            "ok": self.keyGo,
            "save": self.keyGo,
            "green": self.keyGo,
            "red": self.keyCancel,
            "left": self.keyLeft,
            "right": self.keyRight,
        }, -2)

        self["key_red"] = StaticText(_("Exit"))
        self["key_green"] = StaticText(_("Start Scan"))
        self["description"] = Label("")
        self["lnb_info"] = Label("")  # Label to show current LNB info

        # Initialize ConfigListScreen
        self.list = []
        ConfigListScreen.__init__(self, self.list)
        
        # Don't use onSelectionChanged - causes recursion in Python 3.12
        # Instead, handle UI updates after creating the setup
        
        # Create the config list
        self.createSetup()
        
        # Set up help window for virtual keyboard
        self.setHelpWindowInstances()
        
        # Initialize LNB info display
        self.updateLnbInfo()
    
    def setHelpWindowInstances(self):
        """Set up help window instances for virtual keyboard"""
        self.helpList = []
        self["VirtualKeyBoard"] = Label()
        self["HelpWindow"] = Label()
        
        # Hide them until needed to avoid UI glitches
        self["VirtualKeyBoard"].hide()
        self["HelpWindow"].hide()
        
    def createSetup(self):
        """Create the configuration screen based on current settings"""
        self.list = []
        
        self.list.append(getConfigListEntry(_("Tuner"), self.scan_nims, _("Select which tuner to use")))
        self.list.append(getConfigListEntry(_("Frequency Band"), self.frequency_band, _("Select C-band or Ku-band")))
        
        # Set default frequency based on band if needed
        if self.frequency_band.value == "cband":
            if not self.frequency.value or self.frequency.value > 5000:
                self.frequency.value = 4000
        else:  # kuband
            if not self.frequency.value or self.frequency.value < 5000:
                self.frequency.value = 11300
        
        # Auto-detect LNB type if tuner is selected
        if self.scan_nims.value and self.lnb_type.value == "default":
            # Remember the user's selected C-band LO preference
            saved_cband_lo = self.cband_lo_freq.value
            
            # Run detection
            self.detectLnbType()
            
            # If user had previously explicitly chosen 5750 MHz for C-band, restore it
            # This makes the UI respect user preferences over auto-detection
            if self.frequency_band.value == "cband" and saved_cband_lo == "5750":
                self.cband_lo_freq.value = "5750"
                
        self.list.append(getConfigListEntry(_("Frequency (MHz)"), self.frequency, _("Transponder frequency in MHz")))
        self.list.append(getConfigListEntry(_("Polarization"), self.polarization, _("Transponder polarization")))
        self.list.append(getConfigListEntry(_("Symbol Rate (kS/s)"), self.symbolrate, _("Symbol rate in kSymbol/sec, e.g., 27500")))
        self.list.append(getConfigListEntry(_("System"), self.system, _("DVB-S or DVB-S2")))
        self.list.append(getConfigListEntry(_("Modulation"), self.modulation, _("Signal modulation type")))
        self.list.append(getConfigListEntry(_("FEC"), self.fec, _("Forward Error Correction")))
        self.list.append(getConfigListEntry(_("LNB Selection"), self.lnb_type, _("Select LNB type or enter manual LO")))
        
        # Show appropriate LNB options based on selections
        if self.lnb_type.value == "default":
            if self.frequency_band.value == "cband":
                self.list.append(getConfigListEntry(_("C-band LNB"), self.cband_lo_freq, _("Select C-band LNB type")))
            else:
                self.list.append(getConfigListEntry(_("Ku-band LNB"), self.kuband_lo_freq, _("Select Ku-band LNB type")))
        else:
            self.list.append(getConfigListEntry(_("LNB LO Frequency (MHz)"), self.manual_lo_freq, _("Enter LNB local oscillator frequency in MHz")))
            
        # Add report format selection
        self.list.append(getConfigListEntry(_("Report Format"), self.report_format, _("Select the format for scan results output")))
        
        # Add option to automatically create PidScan directory
        self.list.append(getConfigListEntry(_("Create PidScan Directory"), config.plugins.pidscanner.create_dir, _("Automatically create a PidScan directory for saving results")))
            
        # No longer needed - the option is removed from UI completely
        # self.list.append(getConfigListEntry(_("Stop Enigma2 During Scan"), self.stop_enigma2, _("Stop Enigma2 to free frontend during scan")))
        
        self["config"].list = self.list
        self["config"].l.setList(self.list)
        
    def detectLnbType(self):
        """Auto-detect LNB type based on tuner configuration"""
        if not self.scan_nims.value:
            self["lnb_info"].setText(_("Please select a tuner first"))
            return
            
        nim = int(self.scan_nims.value)
        
        # Find any satellite configured in the tuner
        orbitals = nimmanager.getSatListForNim(nim)
        if not orbitals:
            self["lnb_info"].setText(_("No satellites configured for this tuner"))
            return
            
        orbital_pos = orbitals[0]  # Use first satellite position
        
        # Get satellite configuration
        satconfig = None
        nim_config = nimmanager.getNimConfig(nim)
        
        is_c_band = False
        is_ku_band = False
        lo_freq = 0
        lnb_name = ""
        orbital_name = ""
        
        # Get orbital position name
        orbital_name = nimmanager.getSatName(orbital_pos)
        
        # Check configuration mode
        if nim_config.configMode.value == "advanced":
            # Advanced mode - check satellite configuration
            if orbital_pos in nim_config.advanced.sat.keys():
                satconfig = nim_config.advanced.sat[orbital_pos]
                
                # Check LNB type based on LO frequency
                if satconfig.lnb.value == "c_band":
                    is_c_band = True
                    lo_freq = int(satconfig.lof.value)
                    lnb_name = f"C-band LNB ({lo_freq} MHz)"
                elif satconfig.lnb.value == "universal_lnb":
                    is_ku_band = True
                    lo_freq = 9750  # Default to low band
                    lnb_name = "Universal LNB (9750/10600 MHz)"
                elif satconfig.lnb.value == "user_defined":
                    # User defined - check LO frequency to determine band
                    lo_freq = int(satconfig.lofl.value)
                    lnb_name = f"Custom LNB ({lo_freq} MHz)"
                    if 5000 <= lo_freq <= 6000:
                        is_c_band = True
                    elif 9000 <= lo_freq <= 11000:
                        is_ku_band = True
        
        elif nim_config.configMode.value in ("simple", "equal"):
            # Simple/equal configuration - check LNB type from nimmanager's config
            if nim_config.configMode.value == "simple":
                lnb_type = nim_config.diseqcMode.value
            else:
                lnb_type = "universal_lnb"  # Equal mode typically uses universal LNB
                
            if lnb_type == "c_band":
                is_c_band = True
                # Check for LO frequency in simple mode - should be in the lnbSatCR property
                try:
                    if hasattr(nim_config, 'advanced') and hasattr(nim_config.advanced, 'sat'):
                        # In simple mode, the C-band LO frequency might be in the first satellite setting
                        sat_idx = list(nim_config.advanced.sat.keys())[0] if nim_config.advanced.sat else None
                        if sat_idx and hasattr(nim_config.advanced.sat[sat_idx], 'lof'):
                            lo_freq = int(nim_config.advanced.sat[sat_idx].lof.value)
                    
                    # If we couldn't find it, check if a property exists with LO info
                    if lo_freq == 0 and hasattr(nim_config, 'c_band_lo_freq'):
                        lo_freq = int(nim_config.c_band_lo_freq.value)
                        
                    # Fall back to 5750 if we still don't have a value (since your LNB is 5750)
                    if lo_freq == 0:
                        lo_freq = 5750
                except:
                    # If any error, use 5750 which is your actual LNB
                    lo_freq = 5750
                    
                lnb_name = f"C-band LNB ({lo_freq} MHz)"
            else:
                is_ku_band = True
                lo_freq = 9750  # Default Ku-band LO
                lnb_name = "Universal LNB (9750/10600 MHz)"
        
        # Set information text
        if orbital_name and lnb_name:
            self["lnb_info"].setText(_("Detected: {} at {} - {}").format(
                lnb_name, 
                orbital_name,
                nim_config.configMode.value.capitalize() + " mode"
            ))
        
        # Update UI based on detection
        if is_c_band:
            self.frequency_band.value = "cband"
            if lo_freq == 5150 or lo_freq == 0:
                self.cband_lo_freq.value = "5150"
            elif lo_freq == 5750:
                self.cband_lo_freq.value = "5750"
            else:
                # Unusual C-band LO - switch to manual
                self.lnb_type.value = "manual"
                self.manual_lo_freq.value = lo_freq
                
        elif is_ku_band:
            self.frequency_band.value = "kuband"
            if lo_freq == 9750 or lo_freq == 0:
                self.kuband_lo_freq.value = "9750"
            elif lo_freq == 10600:
                self.kuband_lo_freq.value = "10600" 
            elif lo_freq == 10750:
                self.kuband_lo_freq.value = "10750"
            else:
                # Unusual Ku-band LO - switch to manual
                self.lnb_type.value = "manual"
                self.manual_lo_freq.value = lo_freq
        
    def keySave(self):
        """Save changes and process with setup"""
        self.saveAll()
        self.close(True)

    def keyLeft(self):
        """Handle left key press"""
        ConfigListScreen.keyLeft(self)
        self.handleKeyChange()
    
    def keyRight(self):
        """Handle right key press"""
        ConfigListScreen.keyRight(self)
        self.handleKeyChange()
        
    def handleKeyChange(self):
        """Handle changes from key presses without causing recursion"""
        # Get current item
        current = self["config"].getCurrent()
        
        # Check if we need to rebuild the setup
        if current and current[0] in (_("Frequency Band"), _("LNB Selection"), _("Tuner")):
            self.createSetup()
            
        # Always update LNB info
        self.updateLnbInfo()
        
    def updateLnbInfo(self):
        """Update the LNB info display based on current settings"""
        # If we're using a manual LO frequency, show that instead of detected info
        if self.lnb_type.value == "manual":
            band_str = "C-band" if self.frequency_band.value == "cband" else "Ku-band"
            self["lnb_info"].setText(_("Manual LNB setup: {} - {} MHz LO frequency").format(
                band_str, 
                self.manual_lo_freq.value
            ))
        # If using default but with a specific selection
        elif self.lnb_type.value == "default":
            if self.frequency_band.value == "cband":
                lo_freq = self.cband_lo_freq.value
                self["lnb_info"].setText(_("Selected: C-band LNB with {} MHz LO frequency").format(lo_freq))
            else:
                lo_freq = self.kuband_lo_freq.value
                if lo_freq == "9750":
                    lo_desc = "Universal LNB (Low Band)"
                elif lo_freq == "10600":
                    lo_desc = "Universal LNB (High Band)"
                elif lo_freq == "10750":
                    lo_desc = "DBS LNB" 
                else:
                    lo_desc = f"LNB with {lo_freq} MHz"
                self["lnb_info"].setText(_("Selected: Ku-band {} LO frequency").format(lo_desc))
        
    def keyGo(self):
        """Start the scan process"""
        # Get the configured values
        tuner = int(self.scan_nims.value)
        
        # Validate frequency based on band
        frequency = self.frequency.value
        if self.frequency_band.value == "cband":
            if not frequency or frequency < 3400 or frequency > 4200:
                self.showLargeMessageBox(_("Invalid C-band frequency! Please enter a value between 3400-4200 MHz."), MessageBox.TYPE_ERROR)
                return
        else:  # kuband
            if not frequency or frequency < 10700 or frequency > 12750:
                self.showLargeMessageBox(_("Invalid Ku-band frequency! Please enter a value between 10700-12750 MHz."), MessageBox.TYPE_ERROR)
                return
                
        polarity = int(self.polarization.value)
        stop_enigma2 = self.stop_enigma2.value
        
        # Get additional transponder parameters
        # Handle symbol rate carefully as it may be a float
        try:
            symbolrate = int(float(self.symbolrate.value) * 1000)  # Convert from kS/s to symbols/sec
        except:
            symbolrate = 27500000  # Default as fallback
        system = int(self.system.value)
        modulation = int(self.modulation.value)
        fec = int(self.fec.value)
            
        # Determine LO frequency to use
        if self.lnb_type.value == "default":
            if self.frequency_band.value == "cband":
                lo_freq = int(self.cband_lo_freq.value)
            else:
                lo_freq = int(self.kuband_lo_freq.value)
        else:
            lo_freq = self.manual_lo_freq.value
            
        # Show confirmation dialog
        pol_str = "V" if polarity else "H"
        band_str = "C-band" if self.frequency_band.value == "cband" else "Ku-band"
        
        # Get system and FEC text for display
        system_str = "DVB-S" if system == 0 else "DVB-S2"
        mod_str = "QPSK" if modulation == 0 else "8PSK"
        fec_strs = ["Auto", "1/2", "2/3", "3/4", "5/6", "7/8", "8/9", "3/5", "4/5", "9/10"]
        fec_str = fec_strs[fec] if 0 <= fec < len(fec_strs) else "Auto"
        
        message = f"Ready to scan:\n\n" + \
                  f"Frequency: {frequency} MHz {pol_str}\n" + \
                  f"Band: {band_str}\n" + \
                  f"LNB LO: {lo_freq} MHz\n" + \
                  f"Symbol Rate: {symbolrate/1000} kS/s\n" + \
                  f"System: {system_str}\n" + \
                  f"Modulation: {mod_str}\n" + \
                  f"FEC: {fec_str}\n" + \
                  f"Tuner: {tuner}\n\n" + \
                  f"Start scan now?"
                  
        # Use our custom larger MessageBox
        self.showLargeMessageBox(
            message,
            MessageBox.TYPE_YESNO,
            callback=lambda confirmed: self.startScan(confirmed, frequency, polarity, lo_freq, system, symbolrate, modulation, fec, stop_enigma2)
        )
        
    def startScan(self, confirmed, frequency, polarity, lo_freq, system, symbolrate, modulation, fec, stop_enigma2):
        """Start the actual scan process"""
        if not confirmed:
            return
            
        # Save current configuration to make it persistent
        self.saveConfig()
            
        # Log scan parameters for debugging
        with open(f"{PID_DATA_DIR}/pid_scan_debug.txt", "w") as f:
            f.write(f"Scan started at: {strftime('%Y-%m-%d %H:%M:%S')}\n")
            f.write(f"Frequency: {frequency} MHz\n")
            f.write(f"Polarity: {polarity} (0=H, 1=V)\n")
            f.write(f"LO Frequency: {lo_freq} MHz\n")
            f.write(f"System: {system} (0=DVB-S, 1=DVB-S2)\n")
            f.write(f"Symbol Rate: {symbolrate} symbols/sec (from UI: {self.symbolrate.value})\n")
            f.write(f"Modulation: {modulation} (0=QPSK, 1=8PSK)\n")
            f.write(f"FEC: {fec}\n")
            f.write(f"Stop Enigma2: {stop_enigma2}\n")
            f.write(f"Report Format: {self.report_format.value}\n")
            
        # Always run without stopping Enigma2 - we have proper frontend management now
        self.runAnalyzerWithoutStopping(frequency, polarity, lo_freq, system, symbolrate, modulation, fec)
        
    def saveConfig(self):
        """Save the user configuration to make it persistent"""
        # Save band selection
        config.plugins.pidscanner.last_frequency_band.value = self.frequency_band.value
        config.plugins.pidscanner.last_frequency_band.save()
        
        # Save LNB selections
        if self.frequency_band.value == "cband":
            # Only save C-band LO if we're in C-band mode
            if self.lnb_type.value == "default":
                # Save the selected value from dropdown
                config.plugins.pidscanner.last_cband_lo.value = self.cband_lo_freq.value
                config.plugins.pidscanner.last_cband_lo.save()
            else:
                # For manual mode, convert the manual value to a preset if it matches
                lo_freq = str(self.manual_lo_freq.value)
                if lo_freq in ["5150", "5750"]:
                    config.plugins.pidscanner.last_cband_lo.value = lo_freq
                    config.plugins.pidscanner.last_cband_lo.save()
        else:
            # Only save Ku-band LO if we're in Ku-band mode
            if self.lnb_type.value == "default":
                # Save the selected value from dropdown
                config.plugins.pidscanner.last_kuband_lo.value = self.kuband_lo_freq.value
                config.plugins.pidscanner.last_kuband_lo.save()
            else:
                # For manual mode, convert the manual value to a preset if it matches
                lo_freq = str(self.manual_lo_freq.value)
                if lo_freq in ["9750", "10600", "10750"]:
                    config.plugins.pidscanner.last_kuband_lo.value = lo_freq
                    config.plugins.pidscanner.last_kuband_lo.save()
        
        # Save report format
        config.plugins.pidscanner.report_format.value = self.report_format.value
        config.plugins.pidscanner.report_format.save()
        
        # Save create_dir setting (already saved but ensure it's up to date)
        config.plugins.pidscanner.create_dir.save()
        
        # Force save
        config.plugins.pidscanner.save()
        config.plugins.save()
        config.save()
        
    # Simple wrapper for MessageBox that doesn't try to modify its appearance
    # This avoids potential errors with custom MessageBox classes
    def showLargeMessageBox(self, message, messageType=MessageBox.TYPE_INFO, timeout=-1, enable_input=True, callback=None):
        """Show a MessageBox with the given parameters - using larger text"""
        # Format the message with line breaks to make it more readable
        # We can't change the font size, but we can make the text more readable with spacing
        formatted_message = "\n\n" + message.replace("\n", "\n\n") + "\n\n"
        
        # Use the enable_input parameter name for consistency
        if callback:
            self.session.openWithCallback(callback, MessageBox, formatted_message, messageType, timeout, enable_input=enable_input)
        else:
            self.session.open(MessageBox, formatted_message, messageType, timeout, enable_input=enable_input)
    
    # Method removed - we no longer stop/restart Enigma2
            
    def openFrontend(self):
        """Opens and initializes the frontend for the selected tuner"""
        if self.frontend or self.raw_channel:
            self.releaseFrontend()
            
        # Get the DVB resource manager
        res_mgr = eDVBResourceManager.getInstance()
        if not res_mgr:
            print("[PIDScanner] Error: Cannot get resource manager instance")
            return False
            
        # Get selected tuner (frontend) ID
        self.feid = int(self.scan_nims.value)
        
        # Allocate a raw channel for this frontend
        self.raw_channel = res_mgr.allocateRawChannel(self.feid)
        if not self.raw_channel:
            print(f"[PIDScanner] Error: Cannot allocate raw channel for frontend ID {self.feid}")
            return False
            
        # Get the frontend from the raw channel
        self.frontend = self.raw_channel.getFrontend()
        if not self.frontend:
            print(f"[PIDScanner] Error: Cannot get frontend for ID {self.feid}")
            self.raw_channel = None
            return False
            
        return True
        
    def prepareFrontend(self):
        """Prepare the frontend for scanning"""
        # First try to open the frontend
        if not self.openFrontend():
            # If we failed, we might need to stop the current service
            if self.session.nav.getCurrentService():
                # Remember current service
                self.session.postScanService = self.session.nav.getCurrentlyPlayingServiceOrGroup()
                
                # Log what we're doing
                print("[PIDScanner] Stopping current service to free frontend resources")
                
                # Stop current service
                self.session.nav.stopService()
                
                # Short delay to ensure resources are released
                for i in range(3):
                    print(f"[PIDScanner] Waiting for resources... ({i+1}/3)")
                    import time
                    time.sleep(0.5)
                
                # Try to open frontend again
                if not self.openFrontend():
                    # Still cannot open the frontend
                    print("[PIDScanner] Error: Still cannot open frontend after stopping service")
                    
                    # Check what might be using the tuner
                    try:
                        import subprocess
                        result = subprocess.run(['fuser', '-v', f"/dev/dvb/adapter{self.feid//256}/frontend{self.feid%256}"], 
                                             capture_output=True, text=True)
                        if result.stdout:
                            print(f"[PIDScanner] Processes using tuner: {result.stdout}")
                    except:
                        pass
                        
                    return False
            else:
                # No service is running but still cannot open frontend
                print("[PIDScanner] Error: Cannot open frontend, and no service is running")
                
                # Check if the device exists
                dvb_path = f"/dev/dvb/adapter{int(self.scan_nims.value)//256}/frontend{int(self.scan_nims.value)%256}"
                if not os.path.exists(dvb_path):
                    print(f"[PIDScanner] Error: Device not found: {dvb_path}")
                    
                return False
                
        # Create a Tuner instance for this frontend
        self.tuner = Tuner(self.frontend)
        print(f"[PIDScanner] Successfully created tuner instance for frontend {self.feid}")
        
        return True
        
    def releaseFrontend(self):
        """Release the frontend resources"""
        # Delete frontend object
        if hasattr(self, 'frontend') and self.frontend:
            self.frontend = None
            
        # Delete raw channel object
        if hasattr(self, 'raw_channel') and self.raw_channel:
            del self.raw_channel
            self.raw_channel = None
            
        # Delete tuner object
        if hasattr(self, 'tuner') and self.tuner:
            del self.tuner
            self.tuner = None
            
    def runAnalyzerWithoutStopping(self, frequency, polarity, lo_freq, system=1, symbolrate=27500000, modulation=0, fec=0):
        """Run the analyzer script with proper frontend management (without stopping Enigma2)"""
        # Log that we're running with "without stopping" mode
        with open(f"{PID_DATA_DIR}/pid_scan_nostop.txt", "w") as f:
            f.write(f"Running analyzer without stopping Enigma2\n")
            f.write(f"Timestamp: {strftime('%Y-%m-%d %H:%M:%S')}\n")
            f.write(f"Symbol Rate: {symbolrate}\n")
        try:
            # Remember current service
            self.session.postScanService = self.session.nav.getCurrentlyPlayingServiceOrGroup()
            
            # First stop current service and prepare frontend
            if not self.prepareFrontend():
                self.showLargeMessageBox(_("Cannot acquire tuner. It may be in use."), MessageBox.TYPE_ERROR)
                return
                
            # Show a "please wait" dialog with user input enabled to allow canceling
            self.showLargeMessageBox(
                _("Running transponder analyzer...\nPlease wait.\n\nPress OK to continue, or press Exit to cancel."),
                MessageBox.TYPE_INFO,
                timeout=5,  # Auto-close after 5 seconds to avoid getting stuck
                enable_input=True,  # Allow canceling the dialog
                callback=lambda result: self.analyzerFinished(result, frequency, polarity, lo_freq)
            )
            
            # Start the analyzer in a separate thread to avoid freezing the UI
            self.analyzer_timer = eTimer()
            self.analyzer_timer.callback.append(
                lambda: self.runAnalyzerInBackground(frequency, polarity, lo_freq, system, symbolrate, modulation, fec)
            )
            self.analyzer_timer.start(500, True)
            
        except Exception as e:
            self.showLargeMessageBox(_("Error: %s") % str(e), MessageBox.TYPE_ERROR)
            # Release frontend and restore service
            self.releaseFrontend()
            if hasattr(self.session, 'postScanService') and self.session.postScanService:
                self.session.nav.playService(self.session.postScanService)
                
    def get_signal_data(self, adapter=0, frontend=0):
        """Get signal metrics from proc filesystem or hardware directly"""
        signal_metrics = {
            'snr': 0,
            'strength': 0,
            'ber': "0.00000000",
            'status': "UnLocked"
        }
        
        try:
            # First try to read SNR from /proc/stb/frontend/X/snr
            snr_path = f"/proc/stb/frontend/{adapter}/snr"
            if os.path.exists(snr_path):
                with open(snr_path, 'r') as f:
                    snr_value = f.read().strip()
                    if snr_value:
                        # Convert to dB (different boxes report differently)
                        try:
                            snr_num = int(snr_value, 16) if '0x' in snr_value else int(snr_value)
                            # Most boxes use a scale where 0xFFFF is 100%, but convert to dB
                            if snr_num > 1000:  # Probably using 16-bit scale
                                signal_metrics['snr'] = snr_num * 0.01  # Convert to dB approximation
                            else:
                                signal_metrics['snr'] = float(snr_num)
                        except:
                            signal_metrics['snr'] = float(snr_value)
            
            # Next try to read signal strength
            strength_path = f"/proc/stb/frontend/{adapter}/signal_strength"
            if os.path.exists(strength_path):
                with open(strength_path, 'r') as f:
                    strength_value = f.read().strip()
                    if strength_value:
                        try:
                            strength_num = int(strength_value, 16) if '0x' in strength_value else int(strength_value)
                            # Most boxes report 0-65535 range
                            if strength_num > 1000:  # Probably using 16-bit scale
                                signal_metrics['strength'] = strength_num * 100 // 65535
                            else:
                                signal_metrics['strength'] = strength_num
                        except:
                            signal_metrics['strength'] = int(strength_value)
            
            # Try to read BER (bit error rate)
            ber_path = f"/proc/stb/frontend/{adapter}/bit_error_rate"
            if os.path.exists(ber_path):
                with open(ber_path, 'r') as f:
                    ber_value = f.read().strip()
                    if ber_value:
                        try:
                            ber_num = int(ber_value)
                            # Format as scientific notation
                            if ber_num > 0:
                                signal_metrics['ber'] = f"{ber_num:.8f}"
                            else:
                                signal_metrics['ber'] = "0.00000000"
                        except:
                            signal_metrics['ber'] = ber_value
            
            # Check lock status
            status_path = f"/proc/stb/frontend/{adapter}/status"
            if os.path.exists(status_path):
                with open(status_path, 'r') as f:
                    status_value = f.read().strip()
                    if "LOCK" in status_value.upper():
                        signal_metrics['status'] = "Locked"
                    else:
                        signal_metrics['status'] = "UnLocked"
                        
            # If we have SNR and status is not explicitly unlocked, 
            # assume lock if SNR is decent
            if signal_metrics['snr'] > 0 and signal_metrics['status'] == "UnLocked":
                signal_metrics['status'] = "Locked"
                
        except Exception as e:
            print(f"Error reading signal metrics: {e}")
        
        return signal_metrics
        
    def runAnalyzerInBackground(self, frequency, polarity, lo_freq, system=1, symbolrate=27500000, modulation=0, fec=0):
        """Run the analyzer script in the background"""
        try:
            # Set the output directory to the PidScan directory if it exists, or create it if enabled
            global PID_DATA_DIR
            PID_DATA_DIR = get_pid_scan_dir(create=config.plugins.pidscanner.create_dir.value)
            
            # Create a timestamp for file naming
            timestamp = strftime("%Y-%m-%d_%H-%M-%S")
            
            # Get frontend adapter information from our tuner
            if self.frontend and self.feid >= 0:
                adapter_info = {
                    "adapter": self.feid // 256,  # Typically adapter = slot/256
                    "frontend": self.feid % 256   # Typically frontend = slot%256
                }
                
                # Read signal metrics before releasing frontend
                try:
                    signal_metrics = self.get_signal_data(adapter_info['adapter'], adapter_info['frontend'])
                    print(f"Signal metrics: SNR={signal_metrics['snr']}dB, Strength={signal_metrics['strength']}%, Status={signal_metrics['status']}")
                    
                    # Save metrics to a file for the analyzer script to use
                    metrics_file = f"{PID_DATA_DIR}/signal_metrics_{int(time())}.txt"
                    with open(metrics_file, "w") as f:
                        f.write(f"SNR={signal_metrics['snr']}\n")
                        f.write(f"STRENGTH={signal_metrics['strength']}\n")
                        f.write(f"BER={signal_metrics['ber']}\n")
                        f.write(f"STATUS={signal_metrics['status']}\n")
                except Exception as e:
                    print(f"Error getting signal metrics: {e}")
                
                # Important: Release the frontend first!
                # The external script needs direct access to the hardware
                print("Releasing frontend resources before external scan...")
                self.releaseFrontend()
                
                # Save frontend information for the analyzer script
                with open(f"{PID_DATA_DIR}/active_frontend.txt", "w") as f:
                    f.write(f"adapter={adapter_info['adapter']}\n")
                    f.write(f"frontend={adapter_info['frontend']}\n")
                    f.write(f"feid={self.feid}\n")
                    f.write(f"timestamp={int(time())}\n")
                    
                print(f"Using frontend: adapter {adapter_info['adapter']}, frontend {adapter_info['frontend']} (feid={self.feid})")
            else:
                print("Warning: No frontend information available")
                if os.path.exists(f"{PID_DATA_DIR}/active_frontend.txt"):
                    os.remove(f"{PID_DATA_DIR}/active_frontend.txt")
                # Make sure frontend is released anyway
                self.releaseFrontend()
            
            # Command line approach with --no-stop option and report format
            # This tells the script not to stop/restart Enigma2
            # Add the report format option
            report_opt = ""
            if self.report_format.value == "json":
                report_opt = "--json-only"
            elif self.report_format.value == "human_readable":
                report_opt = "--text-only"
            # Default is both formats (no specific option needed)
            
            # Determine the polarization letter (H/V) for file naming
            pol_letter = "H" if int(polarity) == 0 else "V"
            
            # Create result placeholders with better naming that includes timestamp
            output_file = f"{PID_DATA_DIR}/PID_Scan_{frequency}_{pol_letter}_{timestamp}.txt"
            
            cmd = ['/usr/bin/cband_analyzer.sh', '--no-stop']
            if report_opt:
                cmd.append(report_opt)
                
            cmd.extend([str(frequency), str(polarity), str(lo_freq)])
            
            # Add other parameters
            if system != 1 or symbolrate != 27500000 or modulation != 0 or fec != 0:
                # We need more complex approach to pass all parameters
                print("Using manual mode to pass all parameters")
                
                # Create a temporary parameter file
                param_file = f"{PID_DATA_DIR}/scan_params_{int(time())}.txt"
                with open(param_file, "w") as f:
                    f.write(f"{frequency}\n")
                    f.write(f"{polarity}\n")
                    f.write(f"{lo_freq}\n")
                    f.write(f"{system}\n")
                    f.write(f"{symbolrate}\n")
                    f.write(f"{modulation}\n")
                    f.write(f"{fec}\n")
                    # Add report format as the 8th parameter
                    f.write(f"{self.report_format.value}\n")
                
                # Log the output directory
                print(f"Saving scan results to: {output_file}")
                
                # Modify command to use manual mode with input from file - ensure it's non-blocking
                # Run script in background with asynchronous execution
                cmd_str = f"cat {param_file} | /usr/bin/cband_analyzer.sh --no-stop {report_opt} --manual > {output_file} 2>&1 &"
                print(f"Running: {cmd_str}")
                
                # Execute command non-blocking
                os.system(cmd_str)
                
                # Clean up parameter file after a delay
                from threading import Timer
                Timer(2.0, lambda: os.path.exists(param_file) and os.remove(param_file)).start()
            else:
                # Log the output directory
                print(f"Saving scan results to: {output_file}")
                
                # Simple command mode - execute non-blocking
                cmd_str = f"/usr/bin/cband_analyzer.sh --no-stop {report_opt} {frequency} {polarity} {lo_freq} > {output_file} 2>&1 &"
                print(f"Running: {cmd_str}")
                
                # Execute command non-blocking
                os.system(cmd_str)
            
            # This is a dummy success for now, we'll check the output file in the callback
            self.analyzer_done = True
            self.analyzer_result = (True, "Analysis running in background, check logs for results.", output_file)
                
        except Exception as e:
            self.analyzer_result = (False, f"Error starting analyzer: {str(e)}", None)
            
    def analyzerFinished(self, dialog_result, frequency, polarity, lo_freq):
        """Called when the analyzer has finished"""
        # Note: We've already released the frontend resources in runAnalyzerInBackground
        # self.releaseFrontend() - removed to avoid double release
        
        # Restore service
        if hasattr(self.session, 'postScanService') and self.session.postScanService:
            self.session.nav.playService(self.session.postScanService)
            
        # Make this a completely non-blocking callback
        # Show that we're checking results, but don't wait for them
        pol_letter = "H" if int(polarity) == 0 else "V"
        timestamp = strftime("%Y-%m-%d_%H-%M-%S")
        output_file = f"{PID_DATA_DIR}/PID_Scan_{frequency}_{pol_letter}_{timestamp}.txt"
        
        # Find the actual output file, as it might have a slightly different timestamp
        matching_files = []
        try:
            for file in os.listdir(PID_DATA_DIR):
                if file.startswith(f"PID_Scan_{frequency}_{pol_letter}_") and file.endswith(".txt"):
                    file_path = os.path.join(PID_DATA_DIR, file)
                    # Only include files created in the last 30 seconds
                    if time() - os.path.getmtime(file_path) < 30:
                        matching_files.append(file_path)
            
            # Sort by modification time (newest first)
            matching_files.sort(key=lambda x: os.path.getmtime(x), reverse=True)
            
            # If we found a matching file, use it
            if matching_files:
                output_file = matching_files[0]
        except Exception as e:
            print(f"Error finding output file: {e}")
        
        # Create a basic message regardless of result availability
        message = _("Scan process started in background.\n\n") + \
                  f"Frequency: {frequency} MHz {pol_letter}\n" + \
                  f"LO Frequency: {lo_freq} MHz\n\n" + \
                  f"Results are being saved to:\n{PID_DATA_DIR}\n\n"
        
        # Try to check if the scan is already done
        # This won't wait for it to complete, just checks if it completed very quickly
        if matching_files:
            try:
                # Check file size of the most recent matching file
                file_size = os.path.getsize(matching_files[0])
                mod_time = os.path.getmtime(matching_files[0])
                time_diff = time() - mod_time
                
                # If file is recent (less than 10 seconds old) and has content
                if file_size > 100 and time_diff < 10:
                    # Extract some info from the file
                    service_count = 0
                    with open(matching_files[0], 'r') as f:
                        content = f.read()
                        # Count services by looking for "Service #" lines
                        service_count = content.count("Service #")
                    
                    message += _("Scan completed successfully!\n") + \
                              _(f"Found {service_count} services. Check the output file for detailed results.")
                else:
                    message += _("Scan in progress. Results will be available shortly in the PidScan directory.")
            except Exception as e:
                print(f"Error checking scan results: {e}")
                message += _("Scan in progress. Check logs for details when completed.")
        else:
            message += _("Scan in progress. Results will be saved to the PidScan directory.")
        
        # Check if we're using PidScan directory and let the user know
        if PID_DATA_DIR != "/tmp":
            message += f"\n\n{_('Results are being saved to:')} {PID_DATA_DIR}"
        else:
            # Suggest creating a PidScan directory
            message += f"\n\n{_('Tip: Create a PidScan directory in /media/usb/, /media/FTA/, or /hdd/ to save scan results there.')}"
        
        # Show a simple confirmation that doesn't require reading files
        self.showLargeMessageBox(message, MessageBox.TYPE_INFO)
            
    def extractScanResults(self, output):
        """Extract meaningful scan results from output"""
        try:
            # Initialize result dictionary
            result = {}
            
            # Try to extract tuning frequency
            tuning_match = re.search(r'Calculated tuning frequency: (\d+) kHz', output)
            if tuning_match:
                result['tuning_freq'] = tuning_match.group(1)
                
            # Try to extract service information if available
            json_start = output.find('{')
            json_end = output.rfind('}') + 1
            
            if json_start >= 0 and json_end > json_start:
                try:
                    json_data = json.loads(output[json_start:json_end])
                    if 'transponder' in json_data and 'services' in json_data['transponder']:
                        services = json_data['transponder']['services']
                        result['service_count'] = len(services)
                        result['services'] = services
                except:
                    pass
                
            return result
        except:
            # On any error, return empty dict
            return {}
            
    def keyCancel(self):
        """Close the plugin"""
        # Make sure to release any frontend resources
        if hasattr(self, 'frontend') and self.frontend:
            self.releaseFrontend()
            
        self.close()
        
def main(session, **kwargs):
    """Main function called by Enigma2 to start the plugin"""
    # Update PID_DATA_DIR to check for PidScan directory first (but don't create it yet)
    global PID_DATA_DIR
    PID_DATA_DIR = get_pid_scan_dir(create=False)
    
    # Check if we're being restarted after a scan
    if fileExists(f"{PID_DATA_DIR}/pid_scan_results.txt"):
        # Read and show the results
        with open(f"{PID_DATA_DIR}/pid_scan_results.txt", "r") as f:
            results = f.read()
            
        # Delete the file so we don't show it again
        os.remove(f"{PID_DATA_DIR}/pid_scan_results.txt")
        
        # Show the results
        session.open(MessageBox, results, MessageBox.TYPE_INFO)
        return
    
    # Check for recent output files
    recent_output_files = False
    try:
        # Find any recent output files (less than 1 hour old)
        output_files = []
        
        # Check all possible PidScan directories
        scan_dirs = ["/tmp"]
        if os.path.exists("/media/usb/PidScan"):
            scan_dirs.append("/media/usb/PidScan")
        if os.path.exists("/media/FTA/PidScan"):
            scan_dirs.append("/media/FTA/PidScan")
        if os.path.exists("/hdd/PidScan"):
            scan_dirs.append("/hdd/PidScan")
            
        for scan_dir in scan_dirs:
            try:
                for file in os.listdir(scan_dir):
                    # Check for both old format and new format file names
                    if (file.startswith("analyzer_output_") or file.startswith("PID_Scan_")) and file.endswith(".txt"):
                        file_path = os.path.join(scan_dir, file)
                        mod_time = os.path.getmtime(file_path)
                        if time() - mod_time < 3600:  # 1 hour in seconds
                            # Extract frequency and polarization from filename
                            if file.startswith("analyzer_output_"):
                                freq_pol = file.replace("analyzer_output_", "").replace(".txt", "")
                            else:
                                # Format is PID_Scan_FREQ_POL_TIMESTAMP.txt
                                parts = file.split('_')
                                if len(parts) >= 4:
                                    freq_pol = f"{parts[2]}_{parts[3]}"
                                else:
                                    freq_pol = file  # Fallback
                                    
                            output_files.append((file_path, freq_pol, mod_time))
            except Exception as e:
                print(f"Error reading directory {scan_dir}: {e}")
        
        # Sort by modification time (newest first)
        output_files.sort(key=lambda x: x[2], reverse=True)
        
        if output_files:
            recent_output_files = True
            
            # Create a message with the recent files
            message = _("Recent PID Scanner results available:\n\n")
            for file_path, freq_pol, mod_time in output_files[:5]:  # Show up to 5 recent files
                file_time = strftime("%H:%M:%S", time.localtime(mod_time))
                message += f"- {freq_pol} ({file_time})\n"
                
            if len(output_files) > 5:
                message += f"\n...and {len(output_files) - 5} more\n"
                
            message += _("\nWould you like to view these results or start a new scan?")
            
            # Ask if user wants to view recent results or start a new scan
            session.openWithCallback(
                lambda answer: show_recent_results(session, answer, output_files[0][0]) if answer else session.open(PIDScanner),
                MessageBox,
                message,
                MessageBox.TYPE_YESNO,
                default=False  # Default to "No" (new scan)
            )
            return
    except Exception as e:
        print(f"Error checking recent output files: {e}")
        
    # Check if there are log files from previous scans to display
    if not recent_output_files and fileExists(f"{PID_DATA_DIR}/analyzer_log.txt"):
        # Read the log file
        try:
            with open(f"{PID_DATA_DIR}/analyzer_log.txt", "r") as f:
                log_content = f.read()
                
            # Try to extract meaningful information
            result_lines = []
            
            # Look for key information lines
            for line in log_content.split('\n'):
                if any(key in line for key in ["frequency", "Frequency", "tuning", "Tuning", "polarization", "Polarization", 
                                              "Success with adapter", "Analysis completed", "services found"]):
                    result_lines.append(line.strip())
                    
            # If we found meaningful lines, show them
            if result_lines:
                message = "Previous scan results:\n\n" + "\n".join(result_lines[-10:])  # Show last 10 lines max
                
                # Ask if user wants to see full log
                session.openWithCallback(
                    lambda answer: show_full_log(session, answer, log_content) if answer else session.open(PIDScanner),
                    MessageBox,
                    message + "\n\nView full log?",
                    MessageBox.TYPE_YESNO
                )
                return
        except:
            # On any error, just open the normal plugin
            pass
            
    # Otherwise start the normal plugin
    session.open(PIDScanner)

def show_recent_results(session, view_results, file_path):
    """Show the most recent scan results or open the main scanner"""
    if view_results:
        try:
            # Read the file
            with open(file_path, "r") as f:
                content = f.read()
                
            # Check if the file is in the new format or old format
            is_new_format = "Transport Stream Analysis Results" in content
            
            # Look for services section
            services_section = ""
            in_services = False
            service_count = 0
            
            # For new format (cband_analyzer.sh direct output)
            if is_new_format:
                # Extract service information 
                service_blocks = []
                current_block = ""
                in_service_block = False
                
                for line in content.split('\n'):
                    if line.startswith("Service #"):
                        if current_block:
                            service_blocks.append(current_block)
                        current_block = line + "\n"
                        in_service_block = True
                        service_count += 1
                    elif in_service_block and line.strip():
                        current_block += line + "\n"
                    elif in_service_block and not line.strip() and "Elementary Streams:" not in line:
                        in_service_block = False
                
                if current_block:
                    service_blocks.append(current_block)
                
                # Only add first 10 services to avoid making the message too large
                services_section = "\n".join(service_blocks[:10])
                if service_count > 10:
                    services_section += f"\n\n...and {service_count - 10} more services"
            else:
                # For old format or JSON output
                for line in content.split('\n'):
                    if "Services:" in line or "Service ID:" in line or "Service #" in line:
                        in_services = True
                        services_section += line + "\n"
                    elif in_services and line.strip():
                        services_section += line + "\n"
                        if line.startswith("  ") or line.startswith("Service #"):  # Service line
                            service_count += 1
                    elif in_services and not line.strip():
                        in_services = False
                
            # Create a summary message
            summary = _("Scan Results Summary:\n\n")
            
            # Extract key parameters
            params = {}
            for line in content.split('\n'):
                for key in ["Frequency:", "Polarization:", "LNB LO frequency:", "Calculated tuning frequency:",
                           "Analysis completed", "Symbol rate:", "Success with adapter", "services found", 
                           "True downlink frequency:", "Signal metrics:"]:
                    if key in line:
                        params[key] = line.strip()
                        break
                        
            # Add parameters to summary
            for key in ["True downlink frequency:", "Frequency:", "Polarization:", "LNB LO frequency:", 
                       "Calculated tuning frequency:", "Symbol rate:"]:
                if key in params:
                    summary += params[key] + "\n"
            
            # Extract signal information if available
            signal_info = ""
            in_signal_info = False
            for line in content.split('\n'):
                if "Signal metrics:" in line:
                    in_signal_info = True
                    signal_info += line + "\n"
                elif in_signal_info and line.strip() and line.startswith("  "):
                    signal_info += line + "\n"
                elif in_signal_info and (not line.strip() or not line.startswith("  ")):
                    in_signal_info = False
            
            if signal_info:
                summary += "\n" + signal_info
                    
            summary += f"\nFound {service_count} services\n\n"
            summary += services_section
            
            # Add information about file location
            summary += f"\n\nFull results saved to:\n{file_path}"
            
            # Show the results
            session.openWithCallback(
                lambda answer: session.open(PIDScanner),
                MessageBox,
                summary,
                MessageBox.TYPE_INFO
            )
        except Exception as e:
            print(f"Error reading results: {e}")
            session.open(PIDScanner)
    else:
        session.open(PIDScanner)
    
def show_full_log(session, show_log, log_content):
    """Show the full log content and then start the main plugin"""
    if show_log:
        session.openWithCallback(
            lambda answer: session.open(PIDScanner),
            MessageBox,
            log_content[:8000] + "\n\n(Log may be truncated if too long)",  # Limit size to avoid very large messages
            MessageBox.TYPE_INFO
        )
    else:
        session.open(PIDScanner)

def Plugins(**kwargs):
    """Return plugin information to Enigma2"""
    return [PluginDescriptor(
        name=_("PID Scanner"), 
        description=_("Advanced DVB-S/S2 Transponder Scanner with PID extraction"), 
        where=PluginDescriptor.WHERE_PLUGINMENU, 
        icon="plugin.png",  # Will use default icon if not found
        fnc=main
    )]