#!/usr/bin/python3
# Adapter module to make enigma-info data available in OE Alliance compatible format

import os
import subprocess

INFO_FILE = "/usr/lib/enigma.info"

# Cache the enigma.info data
enigma_info = {}

def get_info():
    """Read and cache the enigma-info data"""
    global enigma_info
    
    if not enigma_info:
        try:
            with open(INFO_FILE, "r") as f:
                for line in f:
                    if "=" in line:
                        key, value = line.strip().split('=', 1)
                        # Remove quotes if present
                        if value.startswith("'") and value.endswith("'"):
                            value = value[1:-1]
                        enigma_info[key] = value
        except (IOError, OSError):
            # If enigma.info doesn't exist, return empty dict
            pass
    
    return enigma_info

# Backwards compatibility with BoxBranding.so approach
def getValueFromFile(key):
    """Compatibility function for BoxBranding.so approach"""
    func_name = key.replace(' ', '')
    if func_name in globals() and callable(globals()[func_name]):
        return str(globals()[func_name]())
    return "unknown"

# BoxBranding API functions - these functions provide the OE Alliance compatible API

def getMachineBuild():
    info = get_info()
    return info.get("machinebuild", "unknown")

def getMachineProcModel():
    try:
        with open("/proc/stb/info/model", "r") as f:
            return f.read().strip()
    except:
        return getMachineBuild()

def getMachineBrand():
    info = get_info()
    return info.get("displaybrand", "unknown")

def getMachineName():
    info = get_info()
    return info.get("displaymodel", "unknown")

def getMachineMtdKernel():
    info = get_info()
    return info.get("mtdkernel", "")

def getMachineKernelFile():
    info = get_info()
    return info.get("kernelfile", "")

def getMachineMtdRoot():
    info = get_info()
    return info.get("mtdrootfs", "")

def getMachineRootFile():
    info = get_info()
    return info.get("rootfile", "")

def getMachineMKUBIFS():
    info = get_info()
    return info.get("mkubifs", "")

def getMachineUBINIZE():
    info = get_info()
    return info.get("ubinize", "")

def getBoxType():
    info = get_info()
    return info.get("machine", "")

def getBrandOEM():
    info = get_info()
    return info.get("brand", "")

def getOEVersion():
    info = get_info()
    oe_info = info.get("oe", "OE-Alliance 2.0")
    return oe_info.split(" ")[1].strip("()")

def getDriverDate():
    info = get_info()
    return info.get("driversdate", "")

def getImageVersion():
    info = get_info()
    return info.get("imageversion", "")

def getImageBuild():
    info = get_info()
    return info.get("imagebuild", "")

def getImageDevBuild():
    info = get_info()
    return info.get("imagedevbuild", "0")

def getImageType():
    info = get_info()
    return info.get("imagetype", "")

def getImageDistro():
    info = get_info()
    return info.get("distro", "")

def getFeedsUrl():
    info = get_info()
    return info.get("feedsurl", "")

def getImageFolder():
    info = get_info()
    return info.get("imagedir", "")

def getImageFileSystem():
    info = get_info()
    return info.get("imagefs", "")

def getMachineMake():
    info = get_info()
    return info.get("machinebuild", getBoxType())

def getImageArch():
    info = get_info()
    return info.get("architecture", "")

def getDisplayType():
    info = get_info()
    return info.get("displaytype", "")

def getHaveHDMI():
    info = get_info()
    return info.get("hdmi", "False").lower() == "true"

def getHaveYUV():
    info = get_info()
    return info.get("yuv", "False").lower() == "true"

def getHaveRCA():
    info = get_info()
    return info.get("rca", "False").lower() == "true"

def getHaveAVJACK():
    info = get_info()
    return info.get("avjack", "False").lower() == "true"

def getHaveSCART():
    info = get_info()
    return info.get("scart", "False").lower() == "true"

def getHaveSCARTYUV():
    info = get_info()
    return info.get("scartyuv", "False").lower() == "true"

def getHaveDVI():
    info = get_info()
    return info.get("dvi", "False").lower() == "true"

def getHaveHDMIinHD():
    info = get_info()
    return info.get("hdmihdin", "False").lower() == "true"

def getHaveHDMIinFHD():
    info = get_info()
    return info.get("hdmifhdin", "False").lower() == "true"

def getHaveWOL():
    info = get_info()
    return info.get("wol", "False").lower() == "true"

def getHaveWWOL():
    info = get_info()
    return info.get("wwol", "False").lower() == "true"

def getHaveTranscoding():
    info = get_info()
    return info.get("transcoding", "False").lower() == "true"

def getHaveMultiTranscoding():
    info = get_info()
    return info.get("multitranscoding", "False").lower() == "true"

def getHaveCI():
    info = get_info()
    return info.get("ci", "False").lower() == "true"

# Add any other functions that might be needed by plugins

# For direct imports from other modules - this simulates the BoxBranding.so approach
from sys import modules
modules["BoxBranding"] = modules[__name__]

# For debugging and direct command line use
if __name__ == "__main__":
    import sys
    if len(sys.argv) > 1:
        function_name = sys.argv[1]
        if function_name in globals():
            function = globals()[function_name]
            if callable(function):
                print(function())
            else:
                print(function)
        else:
            print(f"Function {function_name} not found")
    else:
        print("Available functions:")
        for name in sorted(globals().keys()):
            if name.startswith("get") and callable(globals()[name]):
                print(f"  {name}")
